@@ -2,7 +2,7 @@
  *	swordapi.h	- This file contains an api usable by non-C++
  *					environments
  *
- * $Id: flatapi.h 1685 2004-12-22 07:05:49Z scribe $
+ * $Id: flatapi.h 1996 2006-11-23 23:12:10Z dglassey $
  *
  * Copyright 1998 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -29,7 +29,7 @@
 
 extern "C" {
 
-#define SWHANDLE long
+#define SWHANDLE intptr_t
 
 
 //-----------------------------------------------------------------
