@@ -3,7 +3,7 @@
  *		  types of modules (e.g. texts, commentaries, maps, lexicons,
  *		  etc.)
  *
- * $Id: swmodule.h 1959 2006-08-28 00:39:56Z scribe $
+ * $Id: swmodule.h 2093 2007-09-28 19:20:22Z scribe $
  *
  * Copyright 1998 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -399,16 +399,16 @@ public:
 	 * @param path path where to create the new module
 	 * @return error
 	 */
-	static signed char createModule(const char *path) { return -1; }
+	static signed char createModule(const char *path);
 
 	/** Modify the current module entry text - only if module isWritable()
 	 */
-	virtual void setEntry(const char *inbuf, long len = -1) { }
+	virtual void setEntry(const char *inbuf, long len= -1);
 
 	/** Link the current module entry to another module entry - only if
 	 *	module isWritable()
 	 */
-	virtual void linkEntry(const SWKey *sourceKey) { }
+	virtual void linkEntry(const SWKey *sourceKey);
 
 	/** Delete current module entry - only if module isWritable()
 	 */
@@ -456,6 +456,14 @@ public:
 		return *this;
 	}
 
+	/** Retrieves a container of render filters associated with this
+	 *	module.
+	 * @return container of render filters
+	 */
+	virtual const FilterList &getRenderFilters() const {
+		return *renderFilters;
+	}
+
 	/** Removes a RenderFilter from this module's renderFilters queue
 	 * @param oldfilter the filter to remove
 	 * @return *this
