@@ -1,6 +1,6 @@
 /******************************************************************************
  *
- * $Id: thmlhtmlhref.h 1688 2005-01-01 04:42:26Z scribe $
+ * $Id: thmlhtmlhref.h 1988 2006-11-06 19:07:21Z scribe $
  *
  * Copyright 1998 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -29,6 +29,7 @@ SWORD_NAMESPACE_START
 /** this filter converts ThML text to HTML text with hrefs
  */
 class SWDLLEXPORT ThMLHTMLHREF : public SWBasicFilter {
+	SWBuf imgPrefix;
 protected:
 	class MyUserData : public BasicFilterUserData {
 	public:
@@ -45,6 +46,8 @@ protected:
 	virtual bool handleToken(SWBuf &buf, const char *token, BasicFilterUserData *userData);
 public:
 	ThMLHTMLHREF();
+	virtual const char *getImagePrefix() { return imgPrefix.c_str(); }
+	virtual void setImagePrefix(const char *newImgPrefix) { imgPrefix = newImgPrefix; }
 };
 SWORD_NAMESPACE_END
 #endif /* _THMLHTMLHREF_H */
