@@ -1,6 +1,6 @@
 /******************************************************************************
  *
- * $Id: thmlosis.h,v 1.6 2003/02/20 07:25:20 scribe Exp $
+ * $Id: thmlosis.h 1804 2005-05-07 06:26:10Z scribe $
  *
  * Copyright 2003 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -28,8 +28,6 @@ SWORD_NAMESPACE_START
 /** this filter converts ThML text to OSIS text
  */
 class SWDLLEXPORT ThMLOSIS : public SWFilter {
-protected:
-	virtual const char *convertToOSIS(const char *, const SWKey *key);
 public:
 	ThMLOSIS();
 	virtual ~ThMLOSIS();
