@@ -1,7 +1,7 @@
 /******************************************************************************
  *  versekey.h - code for class 'versekey'- a standard Biblical verse key
  *
- * $Id: treekeyidx.cpp 1896 2006-02-13 09:29:56Z scribe $
+ * $Id: treekeyidx.cpp 2009 2006-11-25 21:24:43Z dglassey $
  *
  * Copyright 1998 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -159,7 +159,7 @@ bool TreeKeyIdx::nextSibling() {
 
 bool TreeKeyIdx::previousSibling() {
 	TreeNode iterator;
-	__u32 target = currentNode.offset;
+	__s32 target = currentNode.offset;
 	if (currentNode.parent > -1) {
 		getTreeNodeFromIdxOffset(currentNode.parent, &iterator);
 		getTreeNodeFromIdxOffset(iterator.firstChild, &iterator);
@@ -459,26 +459,31 @@ void TreeKeyIdx::saveTreeNode(TreeNode *node) {
 void TreeKeyIdx::setText(const char *ikey) {
 	char *buf = 0;
 	stdstr(&buf, ikey);
-	char *leaf = strtok(buf, "/");
+	SWBuf leaf = strtok(buf, "/");
+	leaf.trim();
 	root();
-	while ((leaf) && (!Error())) {
+	while ((leaf.size()) && (!Error())) {
 		bool ok, inChild = false;
+		error = KEYERR_OUTOFBOUNDS;
 		for (ok = firstChild(); ok; ok = nextSibling()) {
 			inChild = true;
-			if (!stricmp(leaf, getLocalName()))
+			if (leaf == getLocalName()) {
+				error = 0;
 				break;
+			}
 		}
 		leaf = strtok(0, "/");
+		leaf.trim();
 		if (!ok) {
 		    	if (inChild) {	// if we didn't find a matching child node, default to first child
 				parent();
 				firstChild();
 			}
-			if (leaf)
-				error = KEYERR_OUTOFBOUNDS;
-			break;
+			error = KEYERR_OUTOFBOUNDS;
 		}
 	}
+	if (leaf.size())
+		error = KEYERR_OUTOFBOUNDS;
 	delete [] buf;
 	unsnappedKeyText = ikey;
 }
