@@ -66,11 +66,18 @@ void VerseKey::init() {
  *		VerseKey::parse for more detailed information)
  */
 
+VerseKey::VerseKey(const SWKey &ikey) : SWKey(ikey)
+{
+	init();
+	copyFrom(ikey);
+}
+
+
 VerseKey::VerseKey(const SWKey *ikey) : SWKey(*ikey)
 {
 	init();
 	if (ikey)
-		parse();
+		copyFrom(*ikey);
 }
 
 
@@ -81,10 +88,10 @@ VerseKey::VerseKey(const SWKey *ikey) : SWKey(*ikey)
  *		VerseKey::parse for more detailed information)
  */
 
-VerseKey::VerseKey(const char *ikey) : SWKey(ikey)
+VerseKey::VerseKey(const char *ikeyText) : SWKey(ikeyText)
 {
 	init();
-	if (ikey)
+	if (ikeyText)
 		parse();
 }
 
@@ -92,15 +99,48 @@ VerseKey::VerseKey(const char *ikey) : SWKey(ikey)
 VerseKey::VerseKey(VerseKey const &k) : SWKey(k)
 {
 	init();
-	autonorm = k.autonorm;
-	headings = k.headings;
-	testament = k.Testament();
-	book = k.Book();
-	chapter = k.Chapter();
-	verse = k.Verse();
-	if (k.isBoundSet()) {
-		LowerBound(k.LowerBound());
-		UpperBound(k.UpperBound());
+	copyFrom(k);
+}
+
+
+/******************************************************************************
+ * VerseKey::copyFrom - Equates this VerseKey to another VerseKey
+ */
+
+void VerseKey::copyFrom(const VerseKey &ikey) {
+	autonorm = ikey.autonorm;
+	headings = ikey.headings;
+	testament = ikey.Testament();
+	book = ikey.Book();
+	chapter = ikey.Chapter();
+	verse = ikey.Verse();
+	if (ikey.isBoundSet()) {
+		LowerBound(ikey.LowerBound());
+		UpperBound(ikey.UpperBound());
+	}
+}
+
+
+/******************************************************************************
+ * VerseKey::copyFrom - Equates this VerseKey to another SWKey
+ */
+
+void VerseKey::copyFrom(const SWKey &ikey) {
+	// check to see if we can do a more specific copy
+	// plus some optimizations
+	const SWKey *fromKey = &ikey;
+	ListKey *tryList = SWDYNAMIC_CAST(ListKey, fromKey);
+	if (tryList) {
+		SWKey *k = tryList->getElement();
+		if (k) fromKey = k;
+	}
+	VerseKey *tryVerse = SWDYNAMIC_CAST(VerseKey, fromKey);
+	if (tryVerse) {
+		copyFrom(*tryVerse);
+	}
+	else {
+		SWKey::copyFrom(*fromKey);
+		parse();
 	}
 }
 
@@ -329,23 +369,30 @@ void VerseKey::freshtext() const
 
 int VerseKey::getBookAbbrev(const char *iabbr)
 {
-	int loop, diff, abLen, min, max, target, retVal = -1;
+	int diff, abLen, min, max, target, retVal = -1;
 
 	char *abbr = 0;
 
 	StringMgr* stringMgr = StringMgr::getSystemStringMgr();
 	const bool hasUTF8Support = StringMgr::hasUTF8Support();
 
-		// TODO: Why do we loop twice?  once upper and once not?
-		// If you wrote this, please comment.
+	// The first iteration of this loop tries to uppercase
+	// the string. If we still fail to match, we try
+	// matching the string without any toupper logic.
+	// This is useful for, say, Chinese user input
+	// on a system that doesn't properly support
+	// a true Unicode-toupper function (!hasUTF8Support)
 	for (int i = 0; i < 2; i++) {
 		stdstr(&abbr, iabbr, 2);
 		strstrip(abbr);
-		if (!i && hasUTF8Support) { //we have support for UTF-8 handling; we expect UTF-8 encoded locales
-			abbr = stringMgr->upperUTF8(abbr, strlen(abbr)*2);
-		}
-		else if (!i) {
-			abbr = stringMgr->upperLatin1(abbr);
+
+		if (!i) {
+			if (hasUTF8Support) { //we have support for UTF-8 handling; we expect UTF-8 encoded locales
+				stringMgr->upperUTF8(abbr, strlen(abbr)*2);
+			}
+			else {
+				stringMgr->upperLatin1(abbr);
+			}
 		}
 
 		abLen = strlen(abbr);
@@ -354,6 +401,7 @@ int VerseKey::getBookAbbrev(const char *iabbr)
 			min = 0;
 			max = abbrevsCnt;
 
+			// binary search for a match
 			while(1) {
 				target = min + ((max - min) / 2);
 				diff = strncmp(abbr, abbrevs[target].ab, abLen);
@@ -364,6 +412,7 @@ int VerseKey::getBookAbbrev(const char *iabbr)
 				else	max = target;
 			}
 
+			// lets keep backing up till we find the 'first' valid match
 			for (; target > 0; target--) {
 				if (strncmp(abbr, abbrevs[target-1].ab, abLen))
 					break;
@@ -396,17 +445,18 @@ int VerseKey::getBookAbbrev(const char *iabbr)
  */
 
 ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool expandRange) {
-	SWKey textkey;
-
 	char book[2048];
 	char number[2048];
+	*book = 0;
+	*number = 0;
 	int tobook = 0;
 	int tonumber = 0;
 	int chap = -1, verse = -1;
 	int bookno = 0;
-	VerseKey curkey, lBound;
-	curkey.setLocale(getLocale());
+	VerseKey curKey, lBound, lastKey;
+	curKey.setLocale(getLocale());
 	lBound.setLocale(getLocale());
+	lastKey.setLocale(getLocale());
 	int loop;
 	char comma = 0;
 	char dash = 0;
@@ -414,14 +464,12 @@ ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool e
 	int q;
 	ListKey tmpListKey;
 	ListKey internalListKey;
-	SWKey tmpDefaultKey = defaultKey;
 	char lastPartial = 0;
 	bool inTerm = true;
-	int notAllDigits;
+	int notAllDigits = 0;
 
-	curkey.AutoNormalize(0);
-	tmpListKey << tmpDefaultKey;
-	tmpListKey.GetElement()->userData = (void *)buf;
+	curKey.AutoNormalize(0);
+	if (defaultKey) lastKey = defaultKey;
 	
 	while (*buf) {
 		switch (*buf) {
@@ -499,49 +547,49 @@ ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool e
 				if ((!stricmp(book, "V")) || (!stricmp(book, "VER"))) {	// Verse abbrev
 					if (verse == -1) {
 						verse = chap;
-						chap = VerseKey(tmpListKey).Chapter();
+						chap = lastKey.Chapter();
 						*book = 0;
 					}
 				}
 				if ((!stricmp(book, "ch")) || (!stricmp(book, "chap"))) {	// Verse abbrev
-					strcpy(book, VerseKey(tmpListKey).getBookName());
+					strcpy(book, lastKey.getBookName());
 				}
 				bookno = getBookAbbrev(book);
 			}
 			if (((bookno > -1) || (!*book)) && ((*book) || (chap >= 0) || (verse >= 0))) {
 				char partial = 0;
-				curkey.Verse(1);
-				curkey.Chapter(1);
-				curkey.Book(1);
+				curKey.Verse(1);
+				curKey.Chapter(1);
+				curKey.Book(1);
 
 				if (bookno < 0) {
-					curkey.Testament(VerseKey(tmpListKey).Testament());
-					curkey.Book(VerseKey(tmpListKey).Book());
+					curKey.Testament(lastKey.Testament());
+					curKey.Book(lastKey.Book());
 				}
 				else {
-					curkey.Testament(1);
-					curkey.Book(bookno);
+					curKey.Testament(1);
+					curKey.Book(bookno);
 				}
 
 				if (((comma)||((verse < 0)&&(bookno < 0)))&&(!lastPartial)) {
 //				if (comma) {
-					curkey.Chapter(VerseKey(tmpListKey).Chapter());
-					curkey.Verse(chap);  // chap because this is the first number captured
+					curKey.Chapter(lastKey.Chapter());
+					curKey.Verse(chap);  // chap because this is the first number captured
 				}
 				else {
 					if (chap >= 0) {
-						curkey.Chapter(chap);
+						curKey.Chapter(chap);
 					}
 					else {
 						partial++;
-						curkey.Chapter(1);
+						curKey.Chapter(1);
 					}
 					if (verse >= 0) {
-						curkey.Verse(verse);
+						curKey.Verse(verse);
 					}
 					else {
 						partial++;
-						curkey.Verse(1);
+						curKey.Verse(1);
 					}
 				}
 
@@ -549,39 +597,41 @@ ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool e
 				for (q = 0; ((buf[q]) && (buf[q] == ' ')); q++);
 				if ((buf[q] == '-') && (expandRange)) {	// if this is a dash save lowerBound and wait for upper
 					buf+=q;
-					VerseKey newElement;
-					newElement.LowerBound(curkey);
-					newElement.setPosition(TOP);
-					tmpListKey << newElement;
+					lastKey.LowerBound(curKey);
+					lastKey.setPosition(TOP);
+					tmpListKey << lastKey;
 					tmpListKey.GetElement()->userData = (void *)buf;
 				}
 				else {
 					if (!dash) { 	// if last separator was not a dash just add
 						if (expandRange && partial) {
-							VerseKey newElement;
-							newElement.LowerBound(curkey);
+							lastKey.LowerBound(curKey);
 							if (partial > 1)
-								curkey.setPosition(MAXCHAPTER);
+								curKey.setPosition(MAXCHAPTER);
 							if (partial > 0)
-								curkey = MAXVERSE;
-							newElement.UpperBound(curkey);
-							newElement = TOP;
-							tmpListKey << newElement;
+								curKey = MAXVERSE;
+							lastKey.UpperBound(curKey);
+							lastKey = TOP;
+							tmpListKey << lastKey;
 							tmpListKey.GetElement()->userData = (void *)buf;
 						}
 						else {
-							tmpListKey << (const SWKey &)(const SWKey)(const char *)curkey;
+							// we store non-range entries as strings so we don't traverse
+							// maybe we should consider just setting
+							// lowerBound and upperBound to the same value
+							tmpListKey << curKey.getText();
 							tmpListKey.GetElement()->userData = (void *)buf;
+							lastKey = curKey;
 						}
 					}
 					else	if (expandRange) {
 						VerseKey *newElement = SWDYNAMIC_CAST(VerseKey, tmpListKey.GetElement());
 						if (newElement) {
 							if (partial > 1)
-								curkey = MAXCHAPTER;
+								curKey = MAXCHAPTER;
 							if (partial > 0)
-								curkey = MAXVERSE;
-							newElement->UpperBound(curkey);
+								curKey = MAXVERSE;
+							newElement->UpperBound(curKey);
 							*newElement = TOP;
 							tmpListKey.GetElement()->userData = (void *)buf;
 						}
@@ -688,87 +738,85 @@ ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool e
 		if ((!stricmp(book, "V")) || (!stricmp(book, "VER"))) {	// Verse abbrev.
 			if (verse == -1) {
 				verse = chap;
-				chap = VerseKey(tmpListKey).Chapter();
+				chap = lastKey.Chapter();
 				*book = 0;
 			}
 		}
 			
 		if ((!stricmp(book, "ch")) || (!stricmp(book, "chap"))) {	// Verse abbrev
-			strcpy(book, VerseKey(tmpListKey).getBookName());
+			strcpy(book, lastKey.getBookName());
 		}
 		bookno = getBookAbbrev(book);
 	}
 	if (((bookno > -1) || (!*book)) && ((*book) || (chap >= 0) || (verse >= 0))) {
 		char partial = 0;
-		curkey.Verse(1);
-		curkey.Chapter(1);
-		curkey.Book(1);
+		curKey.Verse(1);
+		curKey.Chapter(1);
+		curKey.Book(1);
 
 		if (bookno < 0) {
-			curkey.Testament(VerseKey(tmpListKey).Testament());
-			curkey.Book(VerseKey(tmpListKey).Book());
+			curKey.Testament(lastKey.Testament());
+			curKey.Book(lastKey.Book());
 		}
 		else {
-			curkey.Testament(1);
-			curkey.Book(bookno);
+			curKey.Testament(1);
+			curKey.Book(bookno);
 		}
 
 		if (((comma)||((verse < 0)&&(bookno < 0)))&&(!lastPartial)) {
-//		if (comma) {
-			curkey.Chapter(VerseKey(tmpListKey).Chapter());
-			curkey.Verse(chap);  // chap because this is the first number captured
+			curKey.Chapter(lastKey.Chapter());
+			curKey.Verse(chap);  // chap because this is the first number captured
 		}
 		else {
 			if (chap >= 0) {
-				curkey.Chapter(chap);
+				curKey.Chapter(chap);
 			}
 			else {
 				partial++;
-				curkey.Chapter(1);
+				curKey.Chapter(1);
 			}
 			if (verse >= 0) {
-				curkey.Verse(verse);
+				curKey.Verse(verse);
 			}
 			else {
 				partial++;
-				curkey.Verse(1);
+				curKey.Verse(1);
 			}
 		}
 
 		if ((*buf == '-') && (expandRange)) {	// if this is a dash save lowerBound and wait for upper
-			VerseKey newElement;
-			newElement.LowerBound(curkey);
-			newElement = TOP;
-			tmpListKey << newElement;
+			lastKey.LowerBound(curKey);
+			lastKey = TOP;
+			tmpListKey << lastKey;
 			tmpListKey.GetElement()->userData = (void *)buf;
 		}
 		else {
 			if (!dash) { 	// if last separator was not a dash just add
 				if (expandRange && partial) {
-					VerseKey newElement;
-					newElement.LowerBound(curkey);
+					lastKey.LowerBound(curKey);
 					if (partial > 1)
-						curkey = MAXCHAPTER;
+						curKey = MAXCHAPTER;
 					if (partial > 0)
-						curkey = MAXVERSE;
-					newElement.UpperBound(curkey);
-					newElement = TOP;
-					tmpListKey << newElement;
+						curKey = MAXVERSE;
+					lastKey.UpperBound(curKey);
+					lastKey = TOP;
+					tmpListKey << lastKey;
 					tmpListKey.GetElement()->userData = (void *)buf;
 				}
 				else {
-					tmpListKey << (const SWKey &)(const SWKey)(const char *)curkey;
+					tmpListKey << curKey.getText();
 					tmpListKey.GetElement()->userData = (void *)buf;
+					lastKey = curKey;
 				}
 			}
 			else if (expandRange) {
 				VerseKey *newElement = SWDYNAMIC_CAST(VerseKey, tmpListKey.GetElement());
 				if (newElement) {
 					if (partial > 1)
-						curkey = MAXCHAPTER;
+						curKey = MAXCHAPTER;
 					if (partial > 0)
-						curkey = MAXVERSE;
-					newElement->UpperBound(curkey);
+						curKey = MAXVERSE;
+					newElement->UpperBound(curKey);
 					*newElement = TOP;
 					tmpListKey.GetElement()->userData = (void *)buf;
 				}
@@ -777,7 +825,6 @@ ListKey VerseKey::ParseVerseList(const char *buf, const char *defaultKey, bool e
 	}
 	*book = 0;
 	tmpListKey = TOP;
-	tmpListKey.Remove();	// remove defaultKey
 	internalListKey = tmpListKey;
 	internalListKey = TOP;	// Align internalListKey to first element before passing back;
 
@@ -905,28 +952,6 @@ void VerseKey::initBounds() const
 }
 
 
-/******************************************************************************
- * VerseKey::copyFrom - Equates this VerseKey to another VerseKey
- */
-
-void VerseKey::copyFrom(const VerseKey &ikey) {
-	SWKey::copyFrom(ikey);
-
-	parse();
-}
-
-
-/******************************************************************************
- * VerseKey::copyFrom - Equates this VerseKey to another SWKey
- */
-
-void VerseKey::copyFrom(const SWKey &ikey) {
-	SWKey::copyFrom(ikey);
-
-	parse();
-}
-
-
 /******************************************************************************
  * VerseKey::getText - refreshes keytext before returning if cast to
  *				a (char *) is requested
@@ -1530,7 +1555,7 @@ char **VerseKey::osisbooks[] = { osisotbooks, osisntbooks };
 
 const char *VerseKey::getOSISRef() const {
 	static char buf[5][254];
-	static char loop = 0;
+	static int loop = 0;
 
 	if (loop > 4)
 		loop = 0;
@@ -1541,7 +1566,7 @@ const char *VerseKey::getOSISRef() const {
 		sprintf(buf[loop], "%s.%d", osisbooks[Testament()-1][Book()-1], (int)Chapter());
 	else if (Book())
 		sprintf(buf[loop], "%s", osisbooks[Testament()-1][Book()-1]);
-	else	sprintf(buf[loop], "");
+	else	buf[loop][0] = 0;
 	return buf[loop++];
 }
 
