@@ -2,7 +2,7 @@
  *  swmgr.cpp   - implementaion of class SWMgr used to interact with an install
  *				base of sword modules.
  *
- * $Id: swmgr.cpp 1963 2006-08-30 04:16:40Z scribe $
+ * $Id: swmgr.cpp 2113 2007-10-15 00:16:27Z scribe $
  *
  * Copyright 1998 CrossWire Bible Society (http://www.crosswire.org)
  *	CrossWire Bible Society
@@ -32,9 +32,11 @@
 
 #include <swmgr.h>
 #include <rawtext.h>
+#include <rawtext4.h>
 #include <filemgr.h>
 #include <rawgenbook.h>
 #include <rawcom.h>
+#include <rawcom4.h>
 #include <hrefcom.h>
 #include <rawld.h>
 #include <rawld4.h>
@@ -100,20 +102,12 @@ bool SWMgr::isICU = false;
 #endif
 
 
-bool SWMgr::debug = false;
-
 #ifdef GLOBCONFPATH
 const char *SWMgr::globalConfPath = GLOBCONFPATH;
 #else
 const char *SWMgr::globalConfPath = "/etc/sword.conf:/usr/local/etc/sword.conf";
 #endif
 
-#ifdef _MSC_VER
-#define DEBUGSTR(x)
-#else
-#define DEBUGSTR(x) if (SWMgr::debug) std::cerr << x;
-#endif  
-
 void SWMgr::init() {
 	SWOptionFilter *tmpFilter = 0;
 	configPath  = 0;
@@ -362,33 +356,32 @@ void SWMgr::findConfig(char *configType, char **prefixPath, char **configPath, s
 	SWConfig *sysConf = 0;
 
 	*configType = 0;
-	//debug=1;
 
 	// check for a sysConf passed in to us
-	DEBUGSTR("Checking for provided SWConfig(\"sword.conf\")...");
+	SWLog::getSystemLog()->logDebug("Checking for provided SWConfig(\"sword.conf\")...");
 	if (providedSysConf) {
 		sysConf = providedSysConf;
-		DEBUGSTR("found.\n");
+		SWLog::getSystemLog()->logDebug("found.");
 	}
 	else {
 		// check working directory
-		DEBUGSTR("\nChecking working directory for sword.conf...");
+		SWLog::getSystemLog()->logDebug("Checking working directory for sword.conf...");
 		if (FileMgr::existsFile(".", "sword.conf")) {
-			DEBUGSTR("\nOverriding any systemwide or ~/.sword/ sword.conf with one found in current directory.\n");
+			SWLog::getSystemLog()->logDebug("Overriding any systemwide or ~/.sword/ sword.conf with one found in current directory.");
 			sysConfPath = "./sword.conf";
 		}
 		else {
-			DEBUGSTR("\nChecking working directory for mods.conf...");
+			SWLog::getSystemLog()->logDebug("Checking working directory for mods.conf...");
 			if (FileMgr::existsFile(".", "mods.conf")) {
-				DEBUGSTR("found\n");
+				SWLog::getSystemLog()->logDebug("found.");
 				stdstr(prefixPath, "./");
 				stdstr(configPath, "./mods.conf");
 				return;
 			}
 
-			DEBUGSTR("\nChecking working directory for mods.d...");
+			SWLog::getSystemLog()->logDebug("Checking working directory for mods.d...");
 			if (FileMgr::existsDir(".", "mods.d")) {
-				DEBUGSTR("found\n");
+				SWLog::getSystemLog()->logDebug("found.");
 				stdstr(prefixPath, "./");
 				stdstr(configPath, "./mods.d");
 				*configType = 1;
@@ -396,9 +389,9 @@ void SWMgr::findConfig(char *configType, char **prefixPath, char **configPath, s
 			}
 
 		   // check working directory ../library/
-			DEBUGSTR("\nChecking working directory ../library/ for mods.d...");
+			SWLog::getSystemLog()->logDebug("Checking working directory ../library/ for mods.d...");
 			if (FileMgr::existsDir("../library", "mods.d")) {
-				DEBUGSTR("found\n");
+				SWLog::getSystemLog()->logDebug("found.");
 				stdstr(prefixPath, "../library/");
 				stdstr(configPath, "../library/mods.d");
 				*configType = 1;
@@ -406,27 +399,27 @@ void SWMgr::findConfig(char *configType, char **prefixPath, char **configPath, s
 			}
 
 			// check environment variable SWORD_PATH
-			DEBUGSTR("\nChecking SWORD_PATH...");
+			SWLog::getSystemLog()->logDebug("Checking SWORD_PATH...");
 
 			if (envsworddir != NULL) {
 				
-				DEBUGSTR("found (" << envsworddir << ")\n");
+				SWLog::getSystemLog()->logDebug("found (%s).", envsworddir);
 				path = envsworddir;
 				if ((envsworddir[strlen(envsworddir)-1] != '\\') && (envsworddir[strlen(envsworddir)-1] != '/'))
 					path += "/";
 
-				DEBUGSTR("\nChecking $SWORD_PATH for mods.conf...");
+				SWLog::getSystemLog()->logDebug("Checking $SWORD_PATH for mods.conf...");
 				if (FileMgr::existsFile(path.c_str(), "mods.conf")) {
-					DEBUGSTR("found\n");
+					SWLog::getSystemLog()->logDebug("found.");
 					stdstr(prefixPath, path.c_str());
 					path += "mods.conf";
 					stdstr(configPath, path.c_str());
 					return;
 				}
 
-				DEBUGSTR("\nChecking $SWORD_PATH for mods.d...");
+				SWLog::getSystemLog()->logDebug("Checking $SWORD_PATH for mods.d...");
 				if (FileMgr::existsDir(path.c_str(), "mods.d")) {
-					DEBUGSTR("found\n");
+					SWLog::getSystemLog()->logDebug("found.");
 					stdstr(prefixPath, path.c_str());
 					path += "mods.d";
 					stdstr(configPath, path.c_str());
@@ -438,14 +431,14 @@ void SWMgr::findConfig(char *configType, char **prefixPath, char **configPath, s
 
 			// check for systemwide globalConfPath
 
-			DEBUGSTR("\nParsing " << globalConfPath << "...");
+			SWLog::getSystemLog()->logDebug("Parsing %s...", globalConfPath);
 			char *globPaths = 0;
 			char *gfp;
 			stdstr(&globPaths, globalConfPath);
 			for (gfp = strtok(globPaths, ":"); gfp; gfp = strtok(0, ":")) {
-				DEBUGSTR("\nChecking for " << gfp << "...");
+				SWLog::getSystemLog()->logDebug("Checking for %s...", gfp);
 				if (FileMgr::existsFile(gfp)) {
-					DEBUGSTR("found\n");
+					SWLog::getSystemLog()->logDebug("found.");
 					break;
 				}
 			}
@@ -459,7 +452,7 @@ void SWMgr::findConfig(char *configType, char **prefixPath, char **configPath, s
 					homeDir += "/";
 				homeDir += ".sword/sword.conf";
 				if (FileMgr::existsFile(homeDir)) {
-					DEBUGSTR("\nOverriding any systemwide sword.conf with one found in users home directory.\n");
+					SWLog::getSystemLog()->logDebug("Overriding any systemwide sword.conf with one found in users home directory.");
 					sysConfPath = homeDir;
 				}
 			}
@@ -476,21 +469,21 @@ void SWMgr::findConfig(char *configType, char **prefixPath, char **configPath, s
 			if (((*entry).second.c_str()[strlen((*entry).second.c_str())-1] != '\\') && ((*entry).second.c_str()[strlen((*entry).second.c_str())-1] != '/'))
 				path += "/";
 
-			DEBUGSTR("DataPath in " << sysConfPath << " is set to: " << path);
-			DEBUGSTR("\nChecking for mods.conf in DataPath...");
+			SWLog::getSystemLog()->logDebug("DataPath in %s is set to %s.", sysConfPath.c_str(), path.c_str());
+			SWLog::getSystemLog()->logDebug("Checking for mods.conf in DataPath...");
 			
 			if (FileMgr::existsFile(path.c_str(), "mods.conf")) {
-				DEBUGSTR("found\n");
+				SWLog::getSystemLog()->logDebug("found.");
 				stdstr(prefixPath, path.c_str());
 				path += "mods.conf";
 				stdstr(configPath, path.c_str());
 				*configType = 1;
 			}
 
-			DEBUGSTR("\nChecking for mods.d in DataPath...");
+			SWLog::getSystemLog()->logDebug("Checking for mods.d in DataPath...");
 
 			if (FileMgr::existsDir(path.c_str(), "mods.d")) {
-				DEBUGSTR("found\n");
+				SWLog::getSystemLog()->logDebug("found.");
 				stdstr(prefixPath, path.c_str());
 				path += "mods.d";
 				stdstr(configPath, path.c_str());
@@ -519,25 +512,25 @@ void SWMgr::findConfig(char *configType, char **prefixPath, char **configPath, s
 
 	// check ~/.sword/
 
-	DEBUGSTR("\nChecking home directory for ~/.sword...");
+	SWLog::getSystemLog()->logDebug("Checking home directory for ~/.sword...");
 
 	if (envhomedir != NULL) {
 		path = envhomedir;
 		if ((envhomedir[strlen(envhomedir)-1] != '\\') && (envhomedir[strlen(envhomedir)-1] != '/'))
 			path += "/";
 		path += ".sword/";
-		DEBUGSTR("\n  Checking for " << path << "mods.conf...");
+		SWLog::getSystemLog()->logDebug("  Checking for %smods.conf...", path.c_str());
 		if (FileMgr::existsFile(path.c_str(), "mods.conf")) {
-			DEBUGSTR("found\n");
+			SWLog::getSystemLog()->logDebug("found.");
 			stdstr(prefixPath, path.c_str());
 			path += "mods.conf";
 			stdstr(configPath, path.c_str());
 			return;
 		}
 
-		DEBUGSTR("\n  Checking for " << path << "mods.d...");
+		SWLog::getSystemLog()->logDebug("  Checking for %smods.d...", path.c_str());
 		if (FileMgr::existsDir(path.c_str(), "mods.d")) {
-			DEBUGSTR("found\n");
+			SWLog::getSystemLog()->logDebug("found.");
 			stdstr(prefixPath, path.c_str());
 			path += "mods.d";
 			stdstr(configPath, path.c_str());
@@ -652,9 +645,9 @@ signed char SWMgr::Load() {
 
 	if (!config) {	// If we weren't passed a config object at construction, find a config file
 		if (!configPath) {	// If we weren't passed a config path at construction...
-			DEBUGSTR("LOOKING UP MODULE CONFIGURATION...\n");
+			SWLog::getSystemLog()->logDebug("LOOKING UP MODULE CONFIGURATION...");
 			findConfig(&configType, &prefixPath, &configPath, &augPaths, sysconfig);
-			DEBUGSTR("LOOKING UP MODULE CONFIGURATION COMPLETE.\n\n");
+			SWLog::getSystemLog()->logDebug("LOOKING UP MODULE CONFIGURATION COMPLETE.");
 		}
 		if (configPath) {
 			if (configType)
@@ -724,6 +717,10 @@ SWModule *SWMgr::CreateMod(const char *name, const char *driver, ConfigEntMap &s
 	datapath = prefixPath;
 	if ((prefixPath[strlen(prefixPath)-1] != '\\') && (prefixPath[strlen(prefixPath)-1] != '/'))
 		datapath += "/";
+
+	// DataPath - relative path to data used by module driver.  May be a directory, may be a File.
+	//   Typically not useful by outside world.  See AbsoluteDataPath, PrefixPath, and RelativePrefixPath
+	//   below.
 	misc1 += ((entry = section.find("DataPath")) != section.end()) ? (*entry).second : (SWBuf)"";
 	char *buf = new char [ strlen(misc1.c_str()) + 1 ];
 	char *buf2 = buf;
@@ -733,6 +730,8 @@ SWModule *SWMgr::CreateMod(const char *name, const char *driver, ConfigEntMap &s
 	if (!strncmp(buf2, "./", 2)) { //remove the leading ./ in the module data path to make it look better
 		buf2 += 2;
 	}
+	// PrefixPath - absolute directory path to the repository in which this module was found
+	section["PrefixPath"] = datapath;
 	if (*buf2)
 		datapath += buf2;
 	delete [] buf;
@@ -805,6 +804,10 @@ SWModule *SWMgr::CreateMod(const char *name, const char *driver, ConfigEntMap &s
 		newmod = new RawText(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
 	}
 
+	if (!stricmp(driver, "RawText4")) {
+		newmod = new RawText4(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
+	}
+
 	// backward support old drivers
 	if (!stricmp(driver, "RawGBF")) {
 		newmod = new RawText(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
@@ -814,6 +817,10 @@ SWModule *SWMgr::CreateMod(const char *name, const char *driver, ConfigEntMap &s
 		newmod = new RawCom(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
 	}
 
+	if (!stricmp(driver, "RawCom4")) {
+		newmod = new RawCom4(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
+	}
+
 	if (!stricmp(driver, "RawFiles")) {
 		newmod = new RawFiles(datapath.c_str(), name, description.c_str(), 0, enc, direction, markup, lang.c_str());
 	}
@@ -870,6 +877,15 @@ SWModule *SWMgr::CreateMod(const char *name, const char *driver, ConfigEntMap &s
 				break;
 			}
 		}
+/*
+		SWBuf &rdp = section["RelativeDataPath"];
+		for (int i = rdp.length() - 1; i; i--) {
+			if (rdp[i] == '/') {
+				rdp.setSize(i);
+				break;
+			}
+		}
+*/
 	}
 
 	// if a specific module type is set in the config, use this
