@@ -75,7 +75,7 @@ long SWCom::Index(long iindex) {
 
 
 VerseKey &SWCom::getVerseKey() const {
-	VerseKey *key;
+	VerseKey *key = NULL;
 	// see if we have a VerseKey * or decendant
 	SWTRY {
 		key = SWDYNAMIC_CAST(VerseKey, this->key);
