@@ -10,8 +10,13 @@ const int EntriesBlock::METAENTRYSIZE = 8;
 	// offset(4); size(4);
 
 EntriesBlock::EntriesBlock(const char *iBlock, unsigned long size) {
-	block = (char *)calloc(1, size);
-	memcpy(block, iBlock, size);
+	if (size) {
+		block = (char *)calloc(1, size);
+		memcpy(block, iBlock, size);
+	}
+	else {
+		block = (char *)calloc(1, sizeof(__u32));
+	}
 }
 
 
@@ -142,9 +147,7 @@ void EntriesBlock::removeEntry(int entryIndex) {
 	unsigned long dataSize;
 	getRawData(&dataSize);
 	getMetaEntry(entryIndex, &offset, &size);
-	unsigned long len = size - 1;
 	int count = getCount();
-	unsigned long dataStart = METAHEADERSIZE + (count * METAENTRYSIZE);
 
 	if (!offset)	// already deleted
 		return;
