@@ -21,16 +21,17 @@
 #include <versekey.h>
 #include <swmodule.h>
 #include <url.h>
+#include <stringmgr.h>
 #include <stack>
 
 SWORD_NAMESPACE_START
 
-class OSISHTMLHREF::QuoteStack : public std::stack<const char*> {
+class OSISHTMLHREF::QuoteStack : public std::stack<char *> {
 };
 
 OSISHTMLHREF::MyUserData::MyUserData(const SWModule *module, const SWKey *key) : BasicFilterUserData(module, key) {
 	inBold = false;
-	inName = false;
+	inXRefNote    = false;
 	suspendLevel = 0;
 	quoteStack = new QuoteStack();
 	wordsOfChristStart = "<font color=\"red\"> ";
@@ -38,6 +39,7 @@ OSISHTMLHREF::MyUserData::MyUserData(const SWModule *module, const SWKey *key) :
 	if (module) {
 		osisQToTick = ((!module->getConfigEntry("OSISqToTick")) || (strcmp(module->getConfigEntry("OSISqToTick"), "false")));
 		version = module->Name();
+		BiblicalText = (!strcmp(module->Type(), "Biblical Texts"));
 	}
 	else {
 		osisQToTick = true;	// default
@@ -48,9 +50,9 @@ OSISHTMLHREF::MyUserData::MyUserData(const SWModule *module, const SWKey *key) :
 OSISHTMLHREF::MyUserData::~MyUserData() {
 	// Just in case the quotes are not well formed
 	while (!quoteStack->empty()) {
-		const char *tagData = quoteStack->top();
+		char *tagData = quoteStack->top();
 		quoteStack->pop();
-		delete tagData;
+		delete [] tagData;
 	}
 	delete quoteStack;
 }
@@ -63,26 +65,89 @@ OSISHTMLHREF::OSISHTMLHREF() {
 	setEscapeEnd(";");
 
 	setEscapeStringCaseSensitive(true);
+	setPassThruNumericEscapeString(true);
 
-//   commenting these out.  If someone is sure we shouldn't
-//   convert these since we are outputing to a markup that
-//   recognizes them, then please delete these lines
-//   addEscapeStringSubstitute("amp",  "&");
-//   addEscapeStringSubstitute("apos", "'");
-//   addEscapeStringSubstitute("lt",   "<");
-//   addEscapeStringSubstitute("gt",   ">");
-//   addEscapeStringSubstitute("quot", "\"");
+	addAllowedEscapeString("quot");
+	addAllowedEscapeString("apos");
+	addAllowedEscapeString("amp");
+	addAllowedEscapeString("lt");
+	addAllowedEscapeString("gt");
 
 	setTokenCaseSensitive(true);
 	
 	addTokenSubstitute("lg",  "<br />");
 	addTokenSubstitute("/lg", "<br />");
+
+	morphFirst = false;
 }
 
 // though this might be slightly slower, possibly causing an extra bool check, this is a renderFilter
 // so speed isn't the absolute highest priority, and this is a very minor possible hit
-static inline void outText(const char *t, SWBuf &o, BasicFilterUserData *u) { if (!u->suspendTextPassThru) o += t; }
-static inline void outText(char t, SWBuf &o, BasicFilterUserData *u) { if (!u->suspendTextPassThru) o += t; }
+static inline void outText(const char *t, SWBuf &o, BasicFilterUserData *u) { if (!u->suspendTextPassThru) o += t; else u->lastSuspendSegment += t; }
+static inline void outText(char t, SWBuf &o, BasicFilterUserData *u) { if (!u->suspendTextPassThru) o += t; else u->lastSuspendSegment += t; }
+
+void processLemma(bool suspendTextPassThru, XMLTag &tag, SWBuf &buf) {
+	const char *attrib;
+	const char *val;
+	if ((attrib = tag.getAttribute("lemma"))) {
+		int count = tag.getAttributePartCount("lemma", ' ');
+		int i = (count > 1) ? 0 : -1;		// -1 for whole value cuz it's faster, but does the same thing as 0
+		do {
+			attrib = tag.getAttribute("lemma", i, ' ');
+			if (i < 0) i = 0;	// to handle our -1 condition
+			val = strchr(attrib, ':');
+			val = (val) ? (val + 1) : attrib;
+			SWBuf gh;
+			if(*val == 'G')
+				gh = "Greek";
+			if(*val == 'H')
+				gh = "Hebrew";
+			const char *val2 = val;
+			if ((strchr("GH", *val)) && (isdigit(val[1])))
+				val2++;
+			//if ((!strcmp(val2, "3588")) && (lastText.length() < 1))
+			//	show = false;
+			//else {
+				if (!suspendTextPassThru) {
+					buf.appendFormatted("<small><em>&lt;<a href=\"passagestudy.jsp?action=showStrongs&type=%s&value=%s\">%s</a>&gt;</em></small>",
+							(gh.length()) ? gh.c_str() : "", 
+							URL::encode(val2).c_str(), 
+							val2);
+				}
+			//}
+			
+		} while (++i < count);
+	}
+}
+
+void processMorph(bool suspendTextPassThru, XMLTag &tag, SWBuf &buf) {
+	const char * attrib;
+	const char *val;
+	if ((attrib = tag.getAttribute("morph"))) { // && (show)) {
+		SWBuf savelemma = tag.getAttribute("savlm");
+		//if ((strstr(savelemma.c_str(), "3588")) && (lastText.length() < 1))
+		//	show = false;
+		//if (show) {
+			int count = tag.getAttributePartCount("morph", ' ');
+			int i = (count > 1) ? 0 : -1;		// -1 for whole value cuz it's faster, but does the same thing as 0
+			do {
+				attrib = tag.getAttribute("morph", i, ' ');
+				if (i < 0) i = 0;	// to handle our -1 condition
+				val = strchr(attrib, ':');
+				val = (val) ? (val + 1) : attrib;
+				const char *val2 = val;
+				if ((*val == 'T') && (strchr("GH", val[1])) && (isdigit(val[2])))
+					val2+=2;
+				if (!suspendTextPassThru) {
+					buf.appendFormatted("<small><em>(<a href=\"passagestudy.jsp?action=showMorph&type=%s&value=%s\">%s</a>)</em></small>",
+							URL::encode(tag.getAttribute("morph")).c_str(),
+							URL::encode(val).c_str(), 
+							val2);
+				}
+			} while (++i < count);
+		//}
+	}
+}
 
 bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *userData) {
 	MyUserData *u = (MyUserData *)userData;
@@ -114,72 +179,27 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 
 				const char *attrib;
 				const char *val;
-				if (attrib = tag.getAttribute("xlit")) {
+				if ((attrib = tag.getAttribute("xlit"))) {
 					val = strchr(attrib, ':');
 					val = (val) ? (val + 1) : attrib;
 					outText(" ", buf, u);
 					outText(val, buf, u);
 				}
-				if (attrib = tag.getAttribute("gloss")) {
+				if ((attrib = tag.getAttribute("gloss"))) {
 					val = strchr(attrib, ':');
 					val = (val) ? (val + 1) : attrib;
 					outText(" ", buf, u);
 					outText(val, buf, u);
 				}
-				if (attrib = tag.getAttribute("lemma")) {
-					int count = tag.getAttributePartCount("lemma", ' ');
-					int i = (count > 1) ? 0 : -1;		// -1 for whole value cuz it's faster, but does the same thing as 0
-					do {
-						attrib = tag.getAttribute("lemma", i, ' ');
-						if (i < 0) i = 0;	// to handle our -1 condition
-						val = strchr(attrib, ':');
-						val = (val) ? (val + 1) : attrib;
-						SWBuf gh;
-						if(*val == 'G')
-							gh = "Greek";
-						if(*val == 'H')
-							gh = "Hebrew";
-						const char *val2 = val;
-						if ((strchr("GH", *val)) && (isdigit(val[1])))
-							val2++;
-						//if ((!strcmp(val2, "3588")) && (lastText.length() < 1))
-						//	show = false;
-						//else {
-							if (!u->suspendTextPassThru) {
-								buf.appendFormatted(" <small><em>&lt;<a href=\"passagestudy.jsp?action=showStrongs&type=%s&value=%s\">%s</a>&gt;</em></small> ", 
-										(gh.length()) ? gh.c_str() : "", 
-										URL::encode(val2).c_str(), 
-										val2);
-							}
-						//}
-						
-					} while (++i < count);
+				if (!morphFirst) {
+					processLemma(u->suspendTextPassThru, tag, buf);
+					processMorph(u->suspendTextPassThru, tag, buf);
 				}
-				if (attrib = tag.getAttribute("morph")) { // && (show)) {
-					SWBuf savelemma = tag.getAttribute("savlm");
-					//if ((strstr(savelemma.c_str(), "3588")) && (lastText.length() < 1))
-					//	show = false;
-					//if (show) {
-						int count = tag.getAttributePartCount("morph", ' ');
-						int i = (count > 1) ? 0 : -1;		// -1 for whole value cuz it's faster, but does the same thing as 0
-						do {
-							attrib = tag.getAttribute("morph", i, ' ');
-							if (i < 0) i = 0;	// to handle our -1 condition
-							val = strchr(attrib, ':');
-							val = (val) ? (val + 1) : attrib;
-							const char *val2 = val;
-							if ((*val == 'T') && (strchr("GH", val[1])) && (isdigit(val[2])))
-								val2+=2;
-							if (!u->suspendTextPassThru) {
-								buf.appendFormatted(" <small><em>(<a href=\"passagestudy.jsp?action=showMorph&type=%s&value=%s\">%s</a>)</em></small> ", 
-										URL::encode(tag.getAttribute("morph")).c_str(),
-										URL::encode(val).c_str(), 
-										val2);
-							}
-						} while (++i < count);
-					//}
+				else {
+					processMorph(u->suspendTextPassThru, tag, buf);
+					processLemma(u->suspendTextPassThru, tag, buf);
 				}
-				if (attrib = tag.getAttribute("POS")) {
+				if ((attrib = tag.getAttribute("POS"))) {
 					val = strchr(attrib, ':');
 					val = (val) ? (val + 1) : attrib;
 					outText(" ", buf, u);
@@ -204,7 +224,9 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 
 					if (!strongsMarkup) {	// leave strong's markup notes out, in the future we'll probably have different option filters to turn different note types on or off
 						SWBuf footnoteNumber = tag.getAttribute("swordFootnote");
-						VerseKey *vkey;
+						VerseKey *vkey = NULL;
+						char ch = ((tag.getAttribute("type") && ((!strcmp(tag.getAttribute("type"), "crossReference")) || (!strcmp(tag.getAttribute("type"), "x-cross-ref")))) ? 'x':'n');
+						u->inXRefNote = (ch == 'x');
 						// see if we have a VerseKey * or descendant
 						SWTRY {
 							vkey = SWDYNAMIC_CAST(VerseKey, u->key);
@@ -212,20 +234,29 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 						SWCATCH ( ... ) {	}
 						if (vkey) {
 							//printf("URL = %s\n",URL::encode(vkey->getText()).c_str());
-							char ch = ((tag.getAttribute("type") && ((!strcmp(tag.getAttribute("type"), "crossReference")) || (!strcmp(tag.getAttribute("type"), "x-cross-ref")))) ? 'x':'n');
-							buf.appendFormatted("<a href=\"passagestudy.jsp?action=showNote&type=%c&value=%s&module=%s&passage=%s\"><small><sup>*%c</sup></small></a> ", 
+							buf.appendFormatted("<a href=\"passagestudy.jsp?action=showNote&type=%c&value=%s&module=%s&passage=%s\"><small><sup>*%c</sup></small></a>",
 								ch, 
 								URL::encode(footnoteNumber.c_str()).c_str(), 
 								URL::encode(u->version.c_str()).c_str(), 
 								URL::encode(vkey->getText()).c_str(), 
 								ch);
 						}
+						else {
+							buf.appendFormatted("<a href=\"passagestudy.jsp?action=showNote&type=%c&value=%s&module=%s&passage=%s\"><small><sup>*%c</sup></small></a>",
+								ch, 
+								URL::encode(footnoteNumber.c_str()).c_str(), 
+								URL::encode(u->version.c_str()).c_str(), 
+								URL::encode(u->key->getText()).c_str(),  
+								ch);
+						}
 					}
 				}
 				u->suspendTextPassThru = (++u->suspendLevel);
 			}
 			if (tag.isEndTag()) {
 				u->suspendTextPassThru = (--u->suspendLevel);
+				u->inXRefNote = false;
+				u->lastSuspendSegment = ""; // fix/work-around for nasb devineName in note bug
 			}
 		}
 
@@ -245,13 +276,48 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 		}
 
 		// <reference> tag
-		else if (!strcmp(tag.getName(), "reference")) {			
-			if ((!tag.isEndTag()) && (!tag.isEmpty())) {
-				outText("<a href=\"\">", buf, u);
-			}
-			else if (tag.isEndTag()) {
-				outText("</a>", buf, u);
-			}
+		else if (!strcmp(tag.getName(), "reference")) {	
+			if (!u->inXRefNote) {	// only show these if we're not in an xref note				
+				if ((!tag.isEndTag()) && (!tag.isEmpty())) {
+					u->suspendTextPassThru = true;
+				}
+				if (tag.isEndTag()) {
+					if (!u->BiblicalText) {
+						SWBuf refList = tag.getAttribute("passage");
+						if (!refList.length())
+							refList = u->lastTextNode;
+						SWBuf version = tag.getAttribute("version");
+						
+						buf.appendFormatted("&nbsp;<a href=\"passagestudy.jsp?action=showRef&type=scripRef&value=%s&module=%s\">",
+							(refList.length()) ? URL::encode(refList.c_str()).c_str() : "", 
+							(version.length()) ? URL::encode(version.c_str()).c_str() : "");
+						buf += u->lastTextNode.c_str();
+						buf += "</a>&nbsp;";
+					}
+					else {
+						SWBuf footnoteNumber = tag.getAttribute("swordFootnote");
+						VerseKey *vkey = NULL;
+						// see if we have a VerseKey * or descendant
+						SWTRY {
+							vkey = SWDYNAMIC_CAST(VerseKey, u->key);
+						}
+						SWCATCH ( ... ) {}
+						if (vkey) {
+							// leave this special osis type in for crossReference notes types?  Might thml use this some day? Doesn't hurt.
+							//buf.appendFormatted("<a href=\"noteID=%s.x.%s\"><small><sup>*x</sup></small></a> ", vkey->getText(), footnoteNumber.c_str());
+							buf.appendFormatted("<a href=\"passagestudy.jsp?action=showNote&type=x&value=%s&module=%s&passage=%s\"><small><sup>*x</sup></small></a>",
+								URL::encode(footnoteNumber.c_str()).c_str(), 
+								URL::encode(u->version.c_str()).c_str(),
+								URL::encode(vkey->getText()).c_str());
+						
+						}
+					}
+					u->suspendTextPassThru = false;
+				}
+			}/*
+			if (tag.isEndTag()) {
+				u->suspendTextPassThru = false;
+			}*/
 		}
 
 		// <l> poetry, etc
@@ -328,23 +394,16 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 		// divineName  
 		else if (!strcmp(tag.getName(), "divineName")) {
 			if ((!tag.isEndTag()) && (!tag.isEmpty())) {
-				u->inName = true;
 				u->suspendTextPassThru = (++u->suspendLevel);
 			}
 			else if (tag.isEndTag()) {
-				if(u->inName ) {
-					u->inName = false;
-					u->suspendTextPassThru = (--u->suspendLevel);
-					char firstChar = *u->lastTextNode.c_str();
-					const char *name = u->lastTextNode.c_str();
-					++name;
-					outText(firstChar, buf, u);
-					outText("<font size=\"-1\">", buf, u);
-					
-					for(int i=0;i<strlen(name);i++)
-						outText(toupper(name[i]), buf, u);
-					outText("</font>", buf, u);
-				}
+				SWBuf lastText = u->lastSuspendSegment.c_str();
+				u->suspendTextPassThru = (--u->suspendLevel);
+				if (lastText.size()) {
+					toupperstr(lastText);
+					scratch.setFormatted("%c<font size=\"-1\">%s</font>", lastText[0], lastText.c_str()+1);
+					outText(scratch.c_str(), buf, u);
+				}               
 			} 
 		}
 
@@ -388,7 +447,7 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 			SWBuf mark      = tmp;
 
 			// open <q> or <q sID... />
-			if ((!tag.isEmpty()) || (tag.getAttribute("sID"))) {
+			if ((!tag.isEmpty() && !tag.isEndTag()) || (tag.isEmpty() && tag.getAttribute("sID"))) {
 				// if <q> then remember it for the </q>
 				if (!tag.isEmpty()) {
 					char *tagData = 0;
@@ -408,13 +467,13 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 					outText((level % 2) ? '\"' : '\'', buf, u);
 			}
 			// close </q> or <q eID... />
-			else if ((tag.isEndTag()) || (tag.getAttribute("eID"))) {
+			else if ((tag.isEndTag()) || (tag.isEmpty() && tag.getAttribute("eID"))) {
 				// if it is </q> then pop the stack for the attributes
 				if (tag.isEndTag() && !u->quoteStack->empty()) {
-					const char *tagData  = u->quoteStack->top();
+					char *tagData  = u->quoteStack->top();
 					u->quoteStack->pop();
 					XMLTag qTag(tagData);
-					delete tagData;
+					delete [] tagData;
 
 					type    = qTag.getAttribute("type");
 					who     = qTag.getAttribute("who");
@@ -473,10 +532,19 @@ bool OSISHTMLHREF::handleToken(SWBuf &buf, const char *token, BasicFilterUserDat
 			}
 			filepath += src;
 
+			// images become clickable, if the UI supports showImage.
+			outText("<a href=\"passagestudy.jsp?action=showImage&value=", buf, u);
+			outText(URL::encode(filepath.c_str()).c_str(), buf, u);
+			outText("&module=", buf, u);
+			outText(URL::encode(u->version.c_str()).c_str(), buf, u);
+			outText("\">", buf, u);
+
 // we do this because BibleCS looks for this EXACT format for an image tag
-			outText("<image src=\"", buf, u);
+			outText("<image border=0 src=\"", buf, u);
 			outText(filepath, buf, u);
 			outText("\" />", buf, u);
+
+			outText("</a>", buf, u);
 		}
 
 		else {
