@@ -32,7 +32,7 @@ namespace {
 		bool BiblicalText;
 		bool inXRefNote;
 		int suspendLevel;
-		std::stack<const char*> quoteStack;
+		std::stack<char *> quoteStack;
 		SWBuf w;
 		SWBuf version;
 		MyUserData(const SWModule *module, const SWKey *key);
@@ -55,13 +55,13 @@ namespace {
 	MyUserData::~MyUserData() {
 		// Just in case the quotes are not well formed
 		while (!quoteStack.empty()) {
-			const char *tagData = quoteStack.top();
+			char *tagData = quoteStack.top();
 			quoteStack.pop();
-			delete tagData;
+			delete [] tagData;
 		}
 	}
-static inline void outText(const char *t, SWBuf &o, BasicFilterUserData *u) { if (!u->suspendTextPassThru) o += t; }
-static inline void outText(char t, SWBuf &o, BasicFilterUserData *u) { if (!u->suspendTextPassThru) o += t; }
+static inline void outText(const char *t, SWBuf &o, BasicFilterUserData *u) { if (!u->suspendTextPassThru) o += t; else u->lastSuspendSegment += t; }
+static inline void outText(char t, SWBuf &o, BasicFilterUserData *u) { if (!u->suspendTextPassThru) o += t; else u->lastSuspendSegment += t; }
 };
 
 
@@ -162,19 +162,19 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 
 				const char *attrib;
 				const char *val;
-				if (attrib = tag.getAttribute("xlit")) {
+				if ((attrib = tag.getAttribute("xlit"))) {
 					val = strchr(attrib, ':');
 					val = (val) ? (val + 1) : attrib;
 					scratch.setFormatted(" {\\fs15 <%s>}", val);
 					outText(scratch.c_str(), buf, u);
 				}
-				if (attrib = tag.getAttribute("gloss")) {
+				if ((attrib = tag.getAttribute("gloss"))) {
 					val = strchr(attrib, ':');
 					val = (val) ? (val + 1) : attrib;
 					scratch.setFormatted(" {\\fs15 <%s>}", val);
 					outText(scratch.c_str(), buf, u);
 				}
-				if (attrib = tag.getAttribute("lemma")) {
+				if ((attrib = tag.getAttribute("lemma"))) {
 					int count = tag.getAttributePartCount("lemma", ' ');
 					int i = (count > 1) ? 0 : -1;		// -1 for whole value cuz it's faster, but does the same thing as 0
 					do {
@@ -213,7 +213,7 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 						} while (++i < count);
 					}
 				}
-				if (attrib = tag.getAttribute("POS")) {
+				if ((attrib = tag.getAttribute("POS"))) {
 					val = strchr(attrib, ':');
 					val = (val) ? (val + 1) : attrib;
 					scratch.setFormatted(" {\\fs15 <%s>}", val);
@@ -235,7 +235,7 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 							&& (type != "strongsMarkup")		// deprecated
 							) {
 						SWBuf footnoteNumber = tag.getAttribute("swordFootnote");
-						VerseKey *vkey;
+						VerseKey *vkey = NULL;
 						// see if we have a VerseKey * or descendant
 						SWTRY {
 							vkey = SWDYNAMIC_CAST(VerseKey, u->key);
@@ -260,7 +260,7 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 		// <p> paragraph tag
 		else if (!strcmp(tag.getName(), "p")) {
 			if ((!tag.isEndTag()) && (!tag.isEmpty())) {	// non-empty start tag
-				outText("{\\par}", buf, u);
+				outText("{\\fi200\\par}", buf, u);
 			}
 			else if (tag.isEndTag()) {	// end tag
 				outText("{\\par}", buf, u);
@@ -359,7 +359,7 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 			SWBuf mark      = tmp;
 
 			// open <q> or <q sID... />
-			if ((!tag.isEmpty()) || (tag.getAttribute("sID"))) {
+			if ((!tag.isEmpty() && !tag.isEndTag()) || (tag.isEmpty() && tag.getAttribute("sID"))) {
 				// if <q> then remember it for the </q>
 				if (!tag.isEmpty()) {
 					char *tagData = 0;
@@ -382,10 +382,10 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 			else if ((tag.isEndTag()) || (tag.getAttribute("eID"))) {
 				// if it is </q> then pop the stack for the attributes
 				if (tag.isEndTag() && !u->quoteStack.empty()) {
-					const char *tagData  = u->quoteStack.top();
+					char *tagData  = u->quoteStack.top();
 					u->quoteStack.pop();
 					XMLTag qTag(tagData);
-					delete tagData;
+					delete [] tagData;
 
 					type    = qTag.getAttribute("type");
 					who     = qTag.getAttribute("who");
@@ -448,7 +448,7 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 				u->suspendTextPassThru = (++u->suspendLevel);
 			}
 			else if (tag.isEndTag()) {
-				SWBuf lastText = u->lastTextNode.c_str();
+				SWBuf lastText = u->lastSuspendSegment.c_str();
 				u->suspendTextPassThru = (--u->suspendLevel);
 				if (lastText.size()) {
 					toupperstr(lastText);
@@ -518,4 +518,3 @@ bool OSISRTF::handleToken(SWBuf &buf, const char *token, BasicFilterUserData *us
 
 
 SWORD_NAMESPACE_END
-
