@@ -4,6 +4,8 @@
  *		  etc.)
  */
 
+#include <vector>
+
 #include <sysdata.h>
 #include <swmodule.h>
 #include <utilstr.h>
@@ -13,6 +15,7 @@
 #include <treekeyidx.h>	// KLUDGE for Search
 #include <swoptfilter.h>
 #include <filemgr.h>
+#include <stringmgr.h>
 #ifndef _MSC_VER
 #include <iostream>
 #endif
@@ -36,6 +39,7 @@ using namespace lucene::queryParser;
 using namespace lucene::search;
 #endif
 
+using std::vector;
 
 SWORD_NAMESPACE_START
 
@@ -403,6 +407,7 @@ void SWModule::decrement(int steps) {
 ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *scope, bool *justCheckIfSupported, void (*percent)(char, void *), void *percentUserData) {
 
 	listKey.ClearList();
+	SWBuf term = istr;
 
 #ifdef USELUCENE
 	SWBuf target = getConfigEntry("AbsoluteDataPath");
@@ -425,9 +430,7 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 	SWKey *searchKey = 0;
 	SWKey *resultKey = CreateKey();
 	regex_t preg;
-	char **words = 0;
-	char *wordBuf = 0;
-	int wordCount = 0;
+	vector<SWBuf> words;
 	const char *sres;
 	terminateSearch = false;
 	char perc = 1;
@@ -564,51 +567,49 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 	}
 #endif
 
+	// some pre-loop processing
+	switch (searchType) {
+
+	// phrase
+	case -1:
+		// let's see if we're told to ignore case.  If so, then we'll touppstr our term
+		if ((flags & REG_ICASE) == REG_ICASE) toupperstr(term);
+		break;
 
 	// multi-word
-	if (searchType == -2) {
-		wordBuf = (char *)calloc(sizeof(char), strlen(istr) + 1);
-		strcpy(wordBuf, istr);
-		words = (char **)calloc(sizeof(char *), 10);
-		int allocWords = 10;
-		words[wordCount] = strtok(wordBuf, " ");
-		while (words[wordCount]) {
-			wordCount++;
-			if (wordCount == allocWords) {
-				allocWords+=10;
-				words = (char **)realloc(words, sizeof(char *)*allocWords);
+	case -2:
+		// let's break the term down into our words vector
+		while (1) {
+			const char *word = term.stripPrefix(' ');
+			if (!word) {
+				words.push_back(term);
+				break;
 			}
-			words[wordCount] = strtok(NULL, " ");
+			words.push_back(word);
 		}
-	}
+		if ((flags & REG_ICASE) == REG_ICASE) {
+			for (unsigned int i = 0; i < words.size(); i++) {
+				toupperstr(words[i]);
+			}
+		}
+		break;
 
 	// entry attributes
-	if (searchType == -3) {
-		wordBuf = (char *)calloc(sizeof(char), strlen(istr) + 1);
-		char *checkSlash = wordBuf;
-		strcpy(wordBuf, istr);
-		words = (char **)calloc(sizeof(char *), 10);
-		int allocWords = 10;
-		while (*checkSlash == '/')
-			words[wordCount++] = checkSlash++;
-		words[wordCount] = strtok(wordBuf, "/");
-		while (words[wordCount]) {
-			wordCount++;
-			if (wordCount == allocWords) {
-				allocWords+=10;
-				words = (char **)realloc(words, sizeof(char *)*allocWords);
+	case -3:
+		// let's break the attribute segs down.  We'll reuse our words vector for each segment
+		while (1) {
+			const char *word = term.stripPrefix('/');
+			if (!word) {
+				words.push_back(term);
+				break;
 			}
-			checkSlash = words[wordCount-1] + (strlen(words[wordCount-1]))+1;
-			while (*checkSlash == '/')
-				words[wordCount++] = checkSlash++;
-			words[wordCount] = strtok(NULL, "/");
-		}
-		for (int i = 0; i < wordCount; i++) {
-			if (words[i][0] == '/')
-				words[i][0] = 0;
+			words.push_back(word);
 		}
+		break;
 	}
 
+
+	// our main loop to iterate the module and find the stuff
 	perc = 5;
 	(*percent)(perc, percentUserData);
 
@@ -645,112 +646,120 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
 		}
 
 		// phrase
-		else if (searchType == -1) {
-			sres = ((flags & REG_ICASE) == REG_ICASE) ? stristr(StripText(), istr) : strstr(StripText(), istr);
-			if (sres) { //it's also in the StripText(), so we have a valid search result item now
-				*resultKey = *getKey();
-				listKey << *resultKey;
-			}
-		}
+		else {
+			SWBuf textBuf;
+			switch (searchType) {
 
-		// multiword
-		else if (searchType == -2) {
-			int loopCount = 0;
-			int foundWords = 0;
-			do {
-				const char* textBuf = ((loopCount == 0)&&(!specialStrips)) ? getRawEntry() : StripText();
-				foundWords = 0;
-				
-				for (int i = 0; i < wordCount; ++i) {
-					sres = ((flags & REG_ICASE) == REG_ICASE) ? stristr(textBuf, words[i]) : strstr(textBuf, words[i]);
-					if (!sres) {
-						break; //for loop
-					}
-					++foundWords;
+			// phrase
+			case -1:
+				textBuf = StripText();
+				if ((flags & REG_ICASE) == REG_ICASE) toupperstr(textBuf);
+				sres = strstr(textBuf.c_str(), term.c_str());
+				if (sres) { //it's also in the StripText(), so we have a valid search result item now
+					*resultKey = *getKey();
+					listKey << *resultKey;
 				}
+				break;
+
+			// multiword
+			case -2: { // enclose our allocations
+				int loopCount = 0;
+				unsigned int foundWords = 0;
+				do {
+					textBuf = ((loopCount == 0)&&(!specialStrips)) ? getRawEntry() : StripText();
+					foundWords = 0;
+					
+					for (unsigned int i = 0; i < words.size(); i++) {
+						if ((flags & REG_ICASE) == REG_ICASE) toupperstr(textBuf);
+						sres = strstr(textBuf.c_str(), words[i].c_str());
+						if (!sres) {
+							break; //for loop
+						}
+						foundWords++;
+					}
+					
+					loopCount++;
+				} while ( (loopCount < 2) && (foundWords == words.size()));
 				
-				++loopCount;
-			} while ( (loopCount < 2) && (foundWords == wordCount));
-			
-			if ((loopCount == 2) && (foundWords == wordCount)) { //we found the right words in both raw and stripped text, which means it's a valid result item
-				*resultKey = *getKey();
-				listKey << *resultKey;
-			}
-		}
+				if ((loopCount == 2) && (foundWords == words.size())) { //we found the right words in both raw and stripped text, which means it's a valid result item
+					*resultKey = *getKey();
+					listKey << *resultKey;
+				}
+				} break;
 
-		// entry attributes
-		else if (searchType == -3) {
-			int i;
-			RenderText();	// force parse
-			AttributeTypeList &entryAttribs = getEntryAttributes();
-			AttributeTypeList::iterator i1Start, i1End;
-			AttributeList::iterator i2Start, i2End;
-			AttributeValue::iterator i3Start, i3End;
+			// entry attributes
+			case -3:
+				RenderText();	// force parse
+				AttributeTypeList &entryAttribs = getEntryAttributes();
+				AttributeTypeList::iterator i1Start, i1End;
+				AttributeList::iterator i2Start, i2End;
+				AttributeValue::iterator i3Start, i3End;
 
-			if ((words[0]) && (words[0][0])) {
-				i1Start = entryAttribs.find(words[0]);
-				i1End = i1Start;
-				if (i1End != entryAttribs.end())
-				i1End++;
-			}
-			else {
-				i1Start = entryAttribs.begin();
-				i1End   = entryAttribs.end();
-			}
-			for (;i1Start != i1End; i1Start++) {
-				if ((words[1]) && (words[1][0])) {
-					i2Start = i1Start->second.find(words[1]);
-					i2End = i2Start;
-					if (i2End != i1Start->second.end())
-						i2End++;
+				if ((words.size()) && (words[0].length())) {
+					i1Start = entryAttribs.find(words[0]);
+					i1End = i1Start;
+					if (i1End != entryAttribs.end())
+					i1End++;
 				}
 				else {
-					i2Start = i1Start->second.begin();
-					i2End   = i1Start->second.end();
+					i1Start = entryAttribs.begin();
+					i1End   = entryAttribs.end();
 				}
-				for (;i2Start != i2End; i2Start++) {
-					if ((words[2]) && (words[2][0])) {
-						i3Start = i2Start->second.find(words[2]);
-						i3End = i3Start;
-						if (i3End != i2Start->second.end())
-							i3End++;
+				for (;i1Start != i1End; i1Start++) {
+					if ((words.size()>1) && (words[1].length())) {
+						i2Start = i1Start->second.find(words[1]);
+						i2End = i2Start;
+						if (i2End != i1Start->second.end())
+							i2End++;
 					}
 					else {
-						i3Start = i2Start->second.begin();
-						i3End   = i2Start->second.end();
+						i2Start = i1Start->second.begin();
+						i2End   = i1Start->second.end();
 					}
-					for (;i3Start != i3End; i3Start++) {
-						if (flags & SEARCHFLAG_MATCHWHOLEENTRY) {
-							bool found = !(((flags & REG_ICASE) == REG_ICASE) ? stricmp(i3Start->second.c_str(), words[3]) : strcmp(i3Start->second.c_str(), words[3]));
-							sres = (found) ? i3Start->second.c_str() : 0;
+					for (;i2Start != i2End; i2Start++) {
+						if ((words.size()>2) && (words[2].length())) {
+							i3Start = i2Start->second.find(words[2]);
+							i3End = i3Start;
+							if (i3End != i2Start->second.end())
+								i3End++;
 						}
 						else {
-							sres = ((flags & REG_ICASE) == REG_ICASE) ? stristr(i3Start->second.c_str(), words[3]) : strstr(i3Start->second.c_str(), words[3]);
+							i3Start = i2Start->second.begin();
+							i3End   = i2Start->second.end();
 						}
-						if (sres) {
-							*resultKey = *getKey();
-							listKey << *resultKey;
+						for (;i3Start != i3End; i3Start++) {
+							if ((words.size()>3) && (words[3].length())) {
+								if (flags & SEARCHFLAG_MATCHWHOLEENTRY) {
+									bool found = !(((flags & REG_ICASE) == REG_ICASE) ? sword::stricmp(i3Start->second.c_str(), words[3]) : strcmp(i3Start->second.c_str(), words[3]));
+									sres = (found) ? i3Start->second.c_str() : 0;
+								}
+								else {
+									sres = ((flags & REG_ICASE) == REG_ICASE) ? stristr(i3Start->second.c_str(), words[3]) : strstr(i3Start->second.c_str(), words[3]);
+								}
+								if (sres) {
+									*resultKey = *getKey();
+									listKey << *resultKey;
+									break;
+								}
+							}
+						}
+						if (i3Start != i3End)
 							break;
-						}
 					}
-					if (i3Start != i3End)
+					if (i2Start != i2End)
 						break;
 				}
-				if (i2Start != i2End)
-					break;
-			}
+				break;
+			} // end switch
 		}
 		(*this)++;
 	}
 	
+
+	// cleaup work
 	if (searchType >= 0)
 		regfree(&preg);
 
-	if (searchType == -2) {
-		free(words);
-		free(wordBuf);
-	}
-
 	setKey(*saveKey);
 
 	if (!saveKey->Persist())
@@ -777,7 +786,7 @@ ListKey &SWModule::search(const char *istr, int searchType, int flags, SWKey *sc
  * ENT:	buf	- buf to massage instead of this modules current text
  * 	len	- max len of buf
  *
- * RET: this module's text at specified key location massaged by Strip filters
+ * RET: this module's text at current key location massaged by Strip filters
  */
 
 const char *SWModule::StripText(const char *buf, int len) {
@@ -790,7 +799,7 @@ const char *SWModule::StripText(const char *buf, int len) {
  *
  * ENT:	buf	- buffer to Render instead of current module position
  *
- * RET: this module's text at specified key location massaged by RenderText filters
+ * RET: this module's text at current key location massaged by RenderText filters
  */
 
  const char *SWModule::RenderText(const char *buf, int len, bool render) {
@@ -831,7 +840,7 @@ const char *SWModule::StripText(const char *buf, int len) {
  *
  * ENT:	tmpKey	- key to use to grab text
  *
- * RET: this module's text at specified key location massaged by RenderFilers
+ * RET: this module's text at current key location massaged by RenderFilers
  */
 
  const char *SWModule::RenderText(SWKey *tmpKey) {
@@ -927,8 +936,7 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 	SWKey *saveKey = 0;
 	SWKey *searchKey = 0;
 	SWKey textkey;
-	char *word = 0;
-	char *wordBuf = 0;
+	SWBuf c;
 
 
 	// turn all filters to default values
@@ -960,29 +968,23 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 		setKey(*searchKey);
 	}
 
-	IndexWriter *writer = NULL;
+	RAMDirectory *ramDir = NULL;
+	IndexWriter *coreWriter = NULL;
+	IndexWriter *fsWriter = NULL;
 	Directory *d = NULL;
  
 	standard::StandardAnalyzer *an = new standard::StandardAnalyzer();
 	SWBuf target = getConfigEntry("AbsoluteDataPath");
+	bool includeKeyInSearch = getConfig().has("SearchOption", "IncludeKeyInSearch");
 	char ch = target.c_str()[strlen(target.c_str())-1];
 	if ((ch != '/') && (ch != '\\'))
 		target.append('/');
 	target.append("lucene");
 	FileMgr::createParent(target+"/dummy");
 
-	if (IndexReader::indexExists(target.c_str())) {
-		d = FSDirectory::getDirectory(target.c_str(), false);
-		if (IndexReader::isLocked(d)) {
-			IndexReader::unlock(d);
-		}
-																		   
-		writer = new IndexWriter( d, an, false);
-	} else {
-		d = FSDirectory::getDirectory(target.c_str(), true);
-		writer = new IndexWriter( d ,an, true);
-	}
-
+	ramDir = new RAMDirectory();
+	coreWriter = new IndexWriter(ramDir, an, true);
+	
 
  
 	char perc = 1;
@@ -1075,7 +1077,14 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 			}
 
 			lucene_utf8towcs(wcharBuffer, keyText, MAX_CONV_SIZE); //keyText must be utf8
-			doc->add( *Field::UnIndexed(_T("key"), wcharBuffer ) );
+			doc->add( *Field::Text(_T("key"), wcharBuffer ) );
+
+			if (includeKeyInSearch) {
+				c = keyText;
+				c += " ";
+				c += content;
+				content = c.c_str();
+			}
 
 			lucene_utf8towcs(wcharBuffer, content, MAX_CONV_SIZE); //content must be utf8
 			doc->add( *Field::UnStored(_T("content"), wcharBuffer) );
@@ -1214,7 +1223,7 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 		if (good) {
 //printf("writing (%s).\n", (const char *)*key);
 //fflush(stdout);
-			writer->addDocument(doc);
+			coreWriter->addDocument(doc);
 		}
 		delete doc;
 
@@ -1222,9 +1231,29 @@ signed char SWModule::createSearchFramework(void (*percent)(char, void *), void
 		err = Error();
 	}
 
-	writer->optimize();
-	writer->close();
-	delete writer;
+	// Optimizing automatically happens with the call to addIndexes
+	//coreWriter->optimize();
+	coreWriter->close();
+
+	if (IndexReader::indexExists(target.c_str())) {
+		d = FSDirectory::getDirectory(target.c_str(), false);
+		if (IndexReader::isLocked(d)) {
+			IndexReader::unlock(d);
+		}
+ 
+		fsWriter = new IndexWriter( d, an, false);
+	} else {
+		d = FSDirectory::getDirectory(target.c_str(), true);
+		fsWriter = new IndexWriter( d ,an, true);
+	}
+
+	Directory *dirs[] = { ramDir, 0 };
+	fsWriter->addIndexes(dirs);
+	fsWriter->close();
+
+	delete ramDir;
+	delete coreWriter;
+	delete fsWriter;
 	delete an;
 
 	// reposition module back to where it was before we were called
@@ -1274,5 +1303,14 @@ void SWModule::filterBuffer(FilterList *filters, SWBuf &buf, SWKey *key) {
 	}
 }
 
+signed char SWModule::createModule(const char*) {
+	return -1;
+}
+
+void SWModule::setEntry(const char*, long) {
+}
+
+void SWModule::linkEntry(const SWKey*) {
+}
 
 SWORD_NAMESPACE_END
