@@ -1,7 +1,7 @@
 /******************************************************************************
 *  swbuf.cpp  - code for SWBuf used as a transport and utility for data buffers
 *
-* $Id: swbuf.cpp 1785 2005-04-30 19:11:12Z scribe $
+* $Id: swbuf.cpp 2102 2007-10-08 18:17:44Z scribe $
 *
 * Copyright 2003 CrossWire Bible Society (http://www.crosswire.org)
 *	CrossWire Bible Society
@@ -80,7 +80,11 @@ void SWBuf::setFormatted(const char *format, ...) {
 	va_list argptr;
 
 	va_start(argptr, format);
+#ifdef NO_VSNPRINTF
 	int len = vsprintf(junkBuf, format, argptr)+1;
+#else
+	int len = vsnprintf(0, 0, format, argptr)+1;
+#endif
 	va_end(argptr);
 	assureSize(len);
 	va_start(argptr, format);
@@ -98,7 +102,7 @@ void SWBuf::append(const char *str, long max) {
 	if (max < 0)
 		max = strlen(str);
 	assureMore(max+1);
-	for (;((*str)&&(max));max--)
+	for (;((max)&&(*str));max--)
 		*end++ = *str++;
 	*end = 0;
 }
@@ -108,7 +112,7 @@ void SWBuf::append(const char *str, long max) {
 */
 void SWBuf::setSize(unsigned long len) {
 	assureSize(len+1);
-	if ((end - buf) < len)
+	if ((unsigned)(end - buf) < len)
 		memset(end, fillByte, len - (end-buf));
 	end = buf + len;
 	*end = 0;
@@ -123,7 +127,11 @@ void SWBuf::appendFormatted(const char *format, ...) {
 	va_list argptr;
 
 	va_start(argptr, format);
+#ifdef NO_VSNPRINTF
 	int len = vsprintf(junkBuf, format, argptr)+1;
+#else
+	int len = vsnprintf(0, 0, format, argptr)+1;
+#endif
 	va_end(argptr);
 	assureMore(len);
 	va_start(argptr, format);
