@@ -5,10 +5,10 @@ OPTIONS="--disable-shared $OPTIONS"
 OPTIONS="--without-conf $OPTIONS"
 OPTIONS="--sysconfdir=/etc $OPTIONS"
 #OPTIONS="--with-vcl $OPTIONS"
-#OPTIONS="--enable-debug $OPTIONS"
+OPTIONS="--enable-debug $OPTIONS"
 #OPTIONS="--enable-profile $OPTIONS"
-OPTIONS="--with-lucene $OPTIONS"
 OPTIONS="--with-icu $OPTIONS"
+#OPTIONS="--without-clucene $OPTIONS"
 #OPTIONS="--without-curl $OPTIONS"
 OPTIONS="--enable-tests $OPTIONS"
 #OPTIONS="--disable-utilities $OPTIONS"
