@@ -4,6 +4,7 @@
 #include <errno.h>
 #include <stdlib.h>
 #include <string.h>
+#include <sys/stat.h>
 
 #ifndef __GNUC__
 #include <io.h>
@@ -11,12 +12,12 @@
 #include <unistd.h>
 #endif
 
+#include <filemgr.h>
 #include <swcipher.h>
 #include <versekey.h>
 #include <rawverse.h>
 #include <swbuf.h>
 
-
 #ifndef NO_SWORD_NAMESPACE
 using namespace sword;
 #endif
@@ -31,7 +32,7 @@ int main(int argc, char **argv) {
 	char *tmpbuf;
 	
 	if (argc != 3) {
-		printf("%d %d\n", sizeof(long), sizeof(unsigned short));
+		printf("%ld %ld\n", sizeof(long), sizeof(unsigned short));
 		fprintf(stderr, "usage: %s <datapath> \"<key>\"\n", argv[0]);
 		exit(1);
 	}
@@ -39,19 +40,15 @@ int main(int argc, char **argv) {
 	rawdrv = new RawVerse(argv[1]);
 	zobj = new SWCipher((unsigned char *)argv[2]);
 
-#ifndef O_BINARY
-#define O_BINARY 0
-#endif
-
 	tmpbuf = new char [ strlen(argv[1]) + 11 ];
 	sprintf(tmpbuf, "%sot.zzz", argv[1]);
-	ofd[0] = open(tmpbuf, O_WRONLY|O_BINARY|O_CREAT);
+	ofd[0] = FileMgr::createPathAndFile(tmpbuf);
 	sprintf(tmpbuf, "%sot.zzz.vss", argv[1]);
-	oxfd[0] = open(tmpbuf, O_WRONLY|O_BINARY|O_CREAT);
+	oxfd[0] = FileMgr::createPathAndFile(tmpbuf);
 	sprintf(tmpbuf, "%snt.zzz", argv[1]);
-	ofd[1] = open(tmpbuf, O_WRONLY|O_BINARY|O_CREAT);
+	ofd[1] = FileMgr::createPathAndFile(tmpbuf);
 	sprintf(tmpbuf, "%snt.zzz.vss", argv[1]);
-	oxfd[1] = open(tmpbuf, O_WRONLY|O_BINARY|O_CREAT);
+	oxfd[1] = FileMgr::createPathAndFile(tmpbuf);
 
 	delete [] tmpbuf;
 
@@ -65,10 +62,10 @@ int main(int argc, char **argv) {
 	key.Headings(1);
 	for (key.Index(0); (!key.Error()); key++) {
 		rawdrv->findOffset(key.Testament(), key.Index(), &offset, &size);
-		printf("%s: OLD offset: %d; size: %d\n", (const char *)key, offset, size);
+		printf("%s: OLD offset: %ld; size: %d\n", (const char *)key, offset, size);
 
 		if ((offset == loffset) && (size == lsize)) {
-			printf("using previous offset,size\n", size);
+			printf("using previous offset,size %d\n", size);
 			offset = lseek(oxfd[key.Testament() - 1], 0, SEEK_CUR);
 			printf("%ld %ld %d \n", offset, lzoffset, lzsize);
 			write(oxfd[key.Testament() - 1], &lzoffset, 4);
