@@ -6,9 +6,9 @@
 #include <string>
 #include <stack>
 #include <iostream>
+#include <fstream>
 
 #include <utilstr.h>
-#include <filemgr.h>
 #include <swmgr.h>
 #include <rawtext.h>
 #include <swbuf.h>
@@ -23,6 +23,10 @@
 #include <zipcomprs.h>
 #include <cipherfil.h>
 
+//#define DEBUG
+
+// Debug for simple transformation stack
+//#define DEBUG2
 
 #ifndef NO_SWORD_NAMESPACE
 using namespace sword;
@@ -33,7 +37,30 @@ using namespace std;
 SWText *module = 0;
 VerseKey *currentVerse = 0;
 char activeOsisID[255];
+char currentOsisID[255];
+char *osisabbrevs[] = {"Gen", "Exod", "Lev", "Num", "Deut", "Josh", "Judg",
+	"Ruth", "1Sam", "2Sam", "1Kgs", "2Kgs", "1Chr", "2Chr", "Ezra", "Neh",
+	"Esth", "Job", "Ps", "Prov", "Eccl", "Song", "Isa", "Jer", "Lam", "Ezek",
+	"Dan", "Hos", "Joel", "Amos", "Obad", "Jonah", "Mic", "Nah", "Hab",
+	"Zeph", "Hag", "Zech", "Mal",
 
+	"Matt", "Mark", "Luke", "John", "Acts", "Rom", "1Cor", "2Cor", "Gal",
+	"Eph", "Phil", "Col", "1Thess", "2Thess", "1Tim", "2Tim", "Titus",
+	"Phlm", "Heb", "Jas", "1Pet", "2Pet", "1John", "2John", "3John",
+	"Jude", "Rev"};
+
+static bool inCanonicalOSISBook = true; // osisID is for a book that is not in Sword's canon
+
+bool isOSISAbbrev(const char *buf) {
+	bool match = false;
+	for (int i = 0; i < 66; i++) {
+		if (!strcmp(buf, osisabbrevs[i])){
+			match = true;
+			break;
+		}
+	}
+	return match;
+}
 
 // remove subverse elements from osisIDs
 void deleteSubverses(SWBuf &buf) {
@@ -59,7 +86,9 @@ bool isKJVRef(const char *buf) {
 	test = buf;
 
 	if (vk.Testament() && vk.Book() && vk.Chapter() && vk.Verse()) { // if we're not a heading
-//		cout << (const char*)vk << " == "  << (const char*)test << endl;
+#ifdef DEBUG
+		cout << (const char*)vk << " == "  << (const char*)test << endl;
+#endif
 		return (vk == test);
 	}
 	else return true;	// no check if we're a heading... Probably bad.
@@ -68,7 +97,9 @@ bool isKJVRef(const char *buf) {
 
 void makeKJVRef(VerseKey &key) {
 	cout << "re-versified " << key;
-//	cout << "\tC" << (int)(key.builtin_books[key.Testament()-1][key.Book()-1].chapmax) << ":V" << (int)(key.builtin_books[key.Testament()-1][key.Book()-1].versemax[key.Chapter()-1]);
+#ifdef DEBUG
+	cout << "\tC" << (int)(key.builtin_books[key.Testament()-1][key.Book()-1].chapmax) << ":V" << (int)(key.builtin_books[key.Testament()-1][key.Book()-1].versemax[key.Chapter()-1]);
+#endif
 	if (key.Chapter() > key.builtin_books[key.Testament()-1][key.Book()-1].chapmax) {
 		key.Chapter(key.builtin_books[key.Testament()-1][key.Book()-1].chapmax);
 		key.Verse(key.builtin_books[key.Testament()-1][key.Book()-1].versemax[key.Chapter()-1]);
@@ -83,60 +114,68 @@ void makeKJVRef(VerseKey &key) {
 void writeEntry(VerseKey &key, SWBuf &text, bool force = false) {
 	static SWBuf activeVerseText;
 	char keyOsisID[255];
-	strcpy(keyOsisID, key.getOSISRef());
 
-	// set keyOsisID to anything that an osisID cannot be.
-	if (force) {
-		strcpy(keyOsisID, "-force");
-	}
+	if (inCanonicalOSISBook) {
+		strcpy(keyOsisID, key.getOSISRef());
 
-	static VerseKey lastKey;
-	lastKey.AutoNormalize(0);
-	lastKey.Headings(1);
-
-	VerseKey saveKey;
-	saveKey.AutoNormalize(0);
-	saveKey.Headings(1);
-	saveKey = key;
-
-	// If we have seen a verse and the supplied one is different then we output the collected one.
-	if (*activeOsisID && strcmp(activeOsisID, keyOsisID)) {
-
-		key = lastKey;
-
-		if (!isKJVRef(key)) {
-			makeKJVRef(key);
+		// set keyOsisID to anything that an osisID cannot be.
+		if (force) {
+			strcpy(keyOsisID, "-force");
 		}
 
-		SWBuf currentText = module->getRawEntry();
-		if (currentText.length()) {
-			cout << "Appending entry: " << key.getOSISRef() << ": " << activeVerseText << endl;
-			activeVerseText = currentText + " " + activeVerseText;
+		static VerseKey lastKey;
+		lastKey.AutoNormalize(0);
+		lastKey.Headings(1);
+
+		VerseKey saveKey;
+		saveKey.AutoNormalize(0);
+		saveKey.Headings(1);
+		saveKey = key;
+
+		// If we have seen a verse and the supplied one is different then we output the collected one.
+		if (*activeOsisID && strcmp(activeOsisID, keyOsisID)) {
+
+			key = lastKey;
+
+			if (!isKJVRef(key)) {
+				makeKJVRef(key);
+			}
+
+			SWBuf currentText = module->getRawEntry();
+			if (currentText.length()) {
+				cout << "Appending entry: " << key.getOSISRef() << ": " << activeVerseText << endl;
+				activeVerseText = currentText + " " + activeVerseText;
+			}
+
+#ifdef DEBUG
+			cout << "Write: " << activeOsisID << ":" << key.getOSISRef() << ": " << activeVerseText << endl;
+#endif
+
+			module->setEntry(activeVerseText);
+			activeVerseText = "";
 		}
 
-//		cout << "Write: " << activeOsisID << ":" << key.getOSISRef() << ": " << activeVerseText << endl;
+		// eliminate leading whitespace on the beginning of each verse and
+		// before we append to current content, since we just added one
+		text.trimStart();
+		if (activeVerseText.length()) {
+			activeVerseText += " ";
+			activeVerseText += text;
+		}
+		else {
+			activeVerseText = text;
+		}
 
-		module->setEntry(activeVerseText);
-		activeVerseText = "";
+		key = saveKey;
+		lastKey = key;
+		strcpy(activeOsisID, keyOsisID);
 	}
-
-	if (activeVerseText.length()) {
-		activeVerseText += " ";
-		activeVerseText += text;
-	}
-	else {
-		activeVerseText = text;
-	}
-
-	key = saveKey;
-	lastKey = key;
-	strcpy(activeOsisID, keyOsisID);
 }
 
 
 void linkToEntry(VerseKey& dest) {
-//	cout << "Verse: " << key << "\n";
-//	cout << "TEXT: " << text << "\n\n";
+	//cout << "Verse: " << key << "\n";
+	//cout << "TEXT: " << text << "\n\n";
 	//SWBuf currentText = module->getRawEntry();
 	//if (currentText.length())
 	//	text = currentText + " " + text;
@@ -165,8 +204,8 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 	static bool inChapterHeader = false;
 
 	// Flags to indicate whether we are in a book, chapter and/or verse
-	static bool inBook = false;
-	static bool inChapter = false;
+	//static bool inBook = false;
+	//static bool inChapter = false;
 	static bool inVerse = true;
 
 	static SWBuf header = "";
@@ -200,25 +239,33 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 	// it belongs to its <div> or <chapter> and is treated as part of its heading
 	// Otherwise if it a title in a chapter before the first the first verse it
 	// is put into the verse as a preverse title.
-	if (!isEndTag && titleDepth == 0 && (!strcmp(tokenName, "title")) && (!typeAttr || (strcmp(typeAttr, "main") && strcmp(typeAttr, "chapter")))) {
+	if (!token->isEmpty() && !isEndTag && titleDepth == 0 && (!strcmp(tokenName, "title")) && (!typeAttr || (strcmp(typeAttr, "main") && strcmp(typeAttr, "chapter")))) {
 		titleOffset = text.length(); //start of the title tag
 		lastTitle = "";
 		inTitle = true;
 		tagStack.push(token);
-//		cout << "push " << token->getName() << endl;
+#ifdef DEBUG
+		cout << currentOsisID << ": push (" << tagStack.size() << ") " << token->getName() << endl;
+#endif
 		titleDepth = tagStack.size();
-		return false; 
+		return false;
 	}
 	// Check titleDepth since titles can be nested. Don't want to quit too early.
 	else if (isEndTag && tagDepth == titleDepth && (!strcmp(tokenName, "title"))) {
 		lastTitle.append(text.c_str() + titleOffset); //<title ...> up to the end </title>
 		lastTitle.append(*token); //</title>
 
-// 		cout << "lastTitle:      " << lastTitle.c_str() << endl;
-// 		cout << "text-lastTitle: " << text.c_str()+titleOffset << endl;
-//		cout << "text:           " << text.c_str() << endl;
+#ifdef DEBUG
+		cout << currentOsisID << ":" << endl;
+ 		cout << "\tlastTitle:      " << lastTitle.c_str() << endl;
+ 		cout << "\ttext-lastTitle: " << text.c_str()+titleOffset << endl;
+		cout << "\ttext:	   " << text.c_str() << endl;
+#endif
 		inTitle = false;
 		titleDepth = 0;
+#ifdef DEBUG
+		cout << currentOsisID << ": pop(" << tagStack.size() << ") " << tagStack.top()->getName() << endl;
+#endif
 		tagStack.pop();
 		return false; // don't add </title> to the text itself
 	}
@@ -232,7 +279,9 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 		// Remember non-empty start tags
 		if (!token->isEmpty()) {
 			tagStack.push(token);
-//			cout << "push " << token->getName() << endl;
+#ifdef DEBUG
+			cout << currentOsisID << ": push (" << tagStack.size() << ") " << token->getName() << endl;
+#endif
 		}
 
 		//-- WITH OSIS ID -------------------------------------------------------------------------
@@ -242,14 +291,17 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 			if ((!strcmp(tokenName, "div")) && (!strcmp(typeAttr, "book"))) {
 				inVerse = false;
 				if (inBookHeader || inChapterHeader) {	// this one should never happen, but just in case
-//					cout << "HEADING ";
+#ifdef DEBUG
+					cout << currentOsisID << ": HEADING ";
+#endif
 					currentVerse->Testament(0);
 					currentVerse->Book(0);
 					currentVerse->Chapter(0);
 					currentVerse->Verse(0);
 					writeEntry(*currentVerse, text);
 				}
-				*currentVerse = token->getAttribute("osisID");
+				strcpy(currentOsisID, token->getAttribute("osisID"));
+				*currentVerse = currentOsisID;
 				currentVerse->Chapter(0);
 				currentVerse->Verse(0);
 				inBookHeader = true;
@@ -260,6 +312,8 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 				chapterDepth = 0;
 				verseDepth = 0;
 
+				inCanonicalOSISBook = isOSISAbbrev(token->getAttribute("osisID"));
+
 				return true;
 			}
 
@@ -269,11 +323,14 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 					 ) {
 				inVerse = false;
 				if (inBookHeader) {
-//					cout << "BOOK HEADING "<< text.c_str() << endl;
+#ifdef DEBUG
+					cout << currentOsisID << ": BOOK HEADING "<< text.c_str() << endl;
+#endif
 					writeEntry(*currentVerse, text);
 				}
 
-				*currentVerse = token->getAttribute("osisID");
+				strcpy(currentOsisID, token->getAttribute("osisID"));
+				*currentVerse = currentOsisID;
 				currentVerse->Verse(0);
 				inBookHeader = false;
 				inChapterHeader = true;
@@ -310,7 +367,7 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 							}
 						}
 						else {
-							cout << "Warning: Bug in code. Could not find title." << endl;
+							cout << currentOsisID << ": Warning: Bug in code. Could not find title." << endl;
 						}
 					}
 					else {
@@ -318,7 +375,9 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 					}
 
 					if (heading.length()) {
-//						cout << "CHAPTER HEADING "<< heading.c_str() << endl;
+#ifdef DEBUG
+						cout << currentOsisID << ": CHAPTER HEADING "<< heading.c_str() << endl;
+#endif
 						writeEntry(*currentVerse, heading);
 					}
 
@@ -341,8 +400,13 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 				}
 
 				lastVerseIDs = currentVerse->ParseVerseList(keyVal);
-				if (lastVerseIDs.Count()) {
+				if (lastVerseIDs.Count() > 1) {
 					*currentVerse = lastVerseIDs.getElement(0)->getText();
+					strcpy(currentOsisID, currentVerse->getOSISRef());
+				}
+				else {
+					strcpy(currentOsisID, keyVal.c_str());
+					*currentVerse = currentOsisID;
 				}
 
 				verseDepth = tagStack.size();
@@ -362,22 +426,31 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 					!strcmp(tokenName, "lb") ||
 					!strcmp(tokenName, "lg")
 					) {
-//					if (token) {
-//						cout << "start token " << *token << ":" << text.c_str() << endl;
-//					}
+#ifdef DEBUG
+					if (token) {
+						cout << currentOsisID << ": appending interverse start token " << *token << ":" << text.c_str() << endl;
+					}
+#endif
 				SWBuf tmp = token->toString();
 				writeEntry(*currentVerse, tmp);
 				return true;
 			}
+#ifdef DEBUG
+			else {
+				if (token) {
+					cout << currentOsisID << ": interverse start token " << *token << ":" << text.c_str() << endl;
+				}
+			}
+#endif
 		}
 	}
 
-//-- END TAG ---------------------------------------------------------------------------------------------
+//-- EMPTY and END TAG ---------------------------------------------------------------------------------------------
 
 	else {
 
 		if (tagStack.empty()) {
-			cout << "tag expected" << endl;
+			cout << currentOsisID << ": tag expected" << endl;
 			exit(1);
 		}
 
@@ -385,12 +458,15 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 		if (!token->isEmpty()) {
 			topToken = tagStack.top();
 			tagDepth = tagStack.size();
-//			cout << "pop " << topToken->getName() << endl;
+#ifdef DEBUG
+			cout << currentOsisID << ": pop(" << tagDepth << ") " << topToken->getName() << endl;
+#endif
 			tagStack.pop();
 
 			if (strcmp(topToken->getName(), tokenName)) {
-				cout << "Error: " << *currentVerse << ": Expected " << topToken->getName() << " found " << tokenName << endl;
-				exit(1);
+				cout << "Error: " << currentOsisID << ": Expected " << topToken->getName() << " found " << tokenName << endl;
+//				exit(1);	// I'm sure this validity check is a good idea, but there's a but somewhere that's killing the converter here.
+						// So I'm disabling this line. Unvalidated OSIS files shouldn't be run through the converter anyway.
 			}
 		}
 
@@ -399,17 +475,22 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 			inVerse = false;
 
 			if (tagDepth != verseDepth) {
-				cout << "Warning verse " << *currentVerse << " is not well formed:(" << verseDepth << "," << tagDepth << ")" << endl;
+				cout << "Warning verse " << currentOsisID << " is not well formed:(" << verseDepth << "," << tagDepth << ")" << endl;
 			}
 
 			if (lastTitle.length()) {
 				const char* end = strchr(lastTitle, '>');
-//				cout << lastTitle << endl;
-//	 			cout << "length=" << int(end+1 - lastTitle.c_str()) << ", tag:" << lastTitle.c_str() << endl;
+#ifdef DEBUG
+				cout << currentOsisID << ":" << endl;
+				cout << "\t" << lastTitle << endl;
+	 			cout << "\tlength=" << int(end+1 - lastTitle.c_str()) << ", tag:" << lastTitle.c_str() << endl;
+#endif
 
 				SWBuf titleTagText;
 				titleTagText.append(lastTitle.c_str(), end+1 - lastTitle.c_str());
-//				cout << "tagText: " << titleTagText.c_str() << endl;;
+#ifdef DEBUG
+				cout << currentOsisID << ": tagText: " << titleTagText.c_str() << endl;;
+#endif
 
 				XMLTag titleTag(titleTagText);
 				titleTag.setAttribute("type", "section");
@@ -459,7 +540,6 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 			return true;
 		}
 		else if (!inTitle && !inVerse && !inBookHeader && !inChapterHeader) {
-//			cout << "End tag not in verse: " << tokenName << "(" << tagDepth << "," << chapterDepth << "," << bookDepth << ")" << endl;
 			// Is this the end of a chapter.
 			if (tagDepth == chapterDepth && (!strcmp(tokenName, "div") || !strcmp(tokenName, "chapter"))) {
 				chapterDepth = 0;
@@ -475,6 +555,14 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 				text = "";
 				return true;
 			}
+			// Or is it the end of an osis document
+			else if (!strcmp(tokenName, "osisText") || !strcmp(tokenName, "osis")) {
+				bookDepth = 0;
+				chapterDepth = 0;
+				verseDepth = 0;
+				text = "";
+				return true;
+			}
 			// OTHER MISC END TAGS WHEN !INVERSE
 			// Test that is between verses, or after the last is appended to the preceeding verse.
 			else if (!strcmp(tokenName, "p") ||
@@ -487,8 +575,14 @@ bool handleToken(SWBuf &text, XMLTag *token) {
 				text.append(*token);
 				writeEntry(*currentVerse, text);
 				text = "";
+#ifdef DEBUG
+				cout << currentOsisID << ": appending interverse end tag: " << tokenName << "(" << tagDepth << "," << chapterDepth << "," << bookDepth << ")" << endl;
+#endif
 				return true;
 			}
+#ifdef DEBUG
+			cout << currentOsisID << ": interverse end tag: " << tokenName << "(" << tagDepth << "," << chapterDepth << "," << bookDepth << ")" << endl;
+#endif
 		}
 	}
 	return false;
@@ -503,6 +597,9 @@ XMLTag* transform(XMLTag* t) {
 	if (!t->isEmpty()) {
 		if (!t->isEndTag()) {
 			tagStack.push(t);
+#ifdef DEBUG2
+			cout << currentOsisID << ": xform push (" << tagStack.size() << ") " << t->getName() << endl;
+#endif
 			// Transform <q> into <q sID=""/> except for <q who="Jesus">
 			if ((!strcmp(t->getName(), "q")) && (!t->getAttribute("who") || strcmp(t->getAttribute("who"), "Jesus"))) {
 				t->setEmpty(true);
@@ -518,6 +615,9 @@ XMLTag* transform(XMLTag* t) {
 		}
 		else {
 			XMLTag *topToken = tagStack.top();
+#ifdef DEBUG2
+			cout << currentOsisID << ": xform pop(" << tagStack.size() << ") " << topToken->getName() << endl;
+#endif
 			tagStack.pop();
 
 			// If we have found an end tag for a <q> that was transformed then transform this one as well.
@@ -538,34 +638,65 @@ XMLTag* transform(XMLTag* t) {
 	return t;
 }
 
+void usage(const char *app, const char *error = 0) {
+
+	if (error) fprintf(stderr, "\n%s: %s\n", app, error);
+
+	fprintf(stderr, "\nusage: %s <output/path> <osisDoc> [OPTIONS]\n", app);
+	fprintf(stderr, "  -a\t\t\t augment module if exists (default is to create new)\n");
+	fprintf(stderr, "  -z\t\t\t use ZIP compression (default no compression)\n");
+	fprintf(stderr, "  -Z\t\t\t use LZSS compression (default no compression)\n");
+	fprintf(stderr, "  -b <2|3|4>\t\t compression block size (default 4):\n");
+	fprintf(stderr, "\t\t\t\t 2 - verse; 3 - chapter; 4 - book\n");
+	fprintf(stderr, "  -c <cipher_key>\t encipher module using supplied key\n");
+	fprintf(stderr, "\t\t\t\t (default no enciphering)\n");
+	exit(-1);
+}
+
 int main(int argc, char **argv) {
 
+	fprintf(stderr, "You are running osis2mod: $Rev: 2090 $\n");
+
 	// Let's test our command line arguments
 	if (argc < 3) {
-		fprintf(stderr, 
-"\nusage: osis2mod <output/path> <osisDoc> [createMod] [compressType [blockType [cipherKey]]]\n");
-		fprintf(stderr, "  createMod   : (default 0): 0 - create  1 - augment\n");
-		fprintf(stderr, "  compressType: (default 0): 0 - no compression  1 - LZSS    2 - Zip\n");
-		fprintf(stderr, "  blockType   : (default 4): 2 - verses  3 - chapters  4 - books\n");
-		fprintf(stderr, "  cipherkey   : ascii string for module encryption\n");
-		exit(-1);
+		usage(*argv);
 	}
 
-	int iType = 4;
-	int compType = 0;
-	string cipherKey = "";
+	// variables for arguments, holding defaults
+	const char* program = argv[0];
+	const char* path    = argv[1];
+	const char* osisDoc = argv[2];
+	int append          = 0;
+	int compType        = 0;
+	int iType           = 4;
+	string cipherKey    = "";
+
 	SWCompress *compressor = 0;
-// 	SWModule *outModule    = 0;
 
-
-	if (argc > 4) {
-		compType = atoi(argv[4]);
-		if (argc > 5) {
-			iType = atoi(argv[5]);
-			if (argc > 6) {
-				cipherKey = argv[6];
-			}
+	for (int i = 3; i < argc; i++) {
+		if (!strcmp(argv[i], "-a")) {
+			append = 1;
 		}
+		else if (!strcmp(argv[i], "-z")) {
+			if (compType) usage(*argv, "Cannot specify both -z and -Z");
+			compType = 2;
+		}
+		else if (!strcmp(argv[i], "-Z")) {
+			if (compType) usage(*argv, "Cannot specify both -z and -Z");
+			compType = 1;
+		}
+		else if (!strcmp(argv[i], "-b")) {
+			if (i+1 < argc) {
+				iType = atoi(argv[++i]);
+				if ((iType >= 2) && (iType <= 4)) continue;
+			}
+			usage(*argv, "-b requires one of <2|3|4>");
+		}
+		else if (!strcmp(argv[i], "-c")) {
+			if (i+1 < argc) cipherKey = argv[++i];
+			else usage(*argv, "-c requires <cipher_key>");
+		}
+		else usage(*argv, (((SWBuf)"Unknown argument: ")+ argv[i]).c_str());
 	}
 
 	switch (compType) {	// these are deleted by zText
@@ -574,41 +705,41 @@ int main(int argc, char **argv) {
 		case 2: compressor = new ZipCompress(); break;
 	}
 
-//	cout << "path: " << argv[1] << " osisDoc: " << argv[2] << " create: " << argv[3] << " compressType: " << compType << " blockType: " << iType << " cipherKey: " << cipherKey.c_str() << "\n";
-//	cout << "";
+#ifdef DEBUG
+	cout << "path: " << path << " osisDoc: " << osisDoc << " create: " << append << " compressType: " << compType << " blockType: " << iType << " cipherKey: " << cipherKey.c_str() << "\n";
+	cout << "";
 //	exit(-3);
+#endif
 
 
-	if ((argc<4)||(!strcmp(argv[3], "0"))) {	// == 0 then create module
+	if (!append) {	// == 0 then create module
 	// Try to initialize a default set of datafiles and indicies at our
 	// datapath location passed to us from the user.
 		if ( compressor ){
-			if ( zText::createModule(argv[1], iType) ){
-				fprintf(stderr, "error: %s: couldn't create module at path: %s \n", argv[0], argv[1]);
+			if ( zText::createModule(path, iType) ){
+				fprintf(stderr, "error: %s: couldn't create module at path: %s \n", program, path);
 				exit(-3);
 			}
 		}
-		else if (RawText::createModule(argv[1])) {
-			fprintf(stderr, "error: %s: couldn't create module at path: %s \n", argv[0], argv[1]);
+		else if (RawText::createModule(path)) {
+			fprintf(stderr, "error: %s: couldn't create module at path: %s \n", program, path);
 			exit(-3);
 		}
 	}
 
 	// Let's see if we can open our input file
-	FileDesc *fd = FileMgr::getSystemFileMgr()->open(argv[2], FileMgr::RDONLY);
-	if (fd->getFd() < 0) {
-		fprintf(stderr, "error: %s: couldn't open input file: %s \n", argv[0], argv[2]);
+	ifstream infile(osisDoc);
+	if (infile.fail()) {
+		fprintf(stderr, "error: %s: couldn't open input file: %s \n", program, osisDoc);
 		exit(-2);
 	}
 
 	// Do some initialization stuff
-	SWBuf buffer;
-
 	if (compressor){
-		module = new zText(argv[1], 0, 0, iType, compressor);
+		module = new zText(path, 0, 0, iType, compressor);
 	}
 	else{
-		module = new RawText(argv[1]);	// open our datapath with our RawText driver.
+		module = new RawText(path);	// open our datapath with our RawText driver.
 	}
 
 	SWFilter *cipherFilter = 0;
@@ -625,6 +756,7 @@ int main(int argc, char **argv) {
 	}
 
 	activeOsisID[0] = '\0';
+	strcpy(currentOsisID,"N/A");
 
 	currentVerse = new VerseKey();
 	currentVerse->AutoNormalize(0);
@@ -635,52 +767,74 @@ int main(int argc, char **argv) {
 
 	(*module) = TOP;
 
-	const char *from;
 	SWBuf token;
 	SWBuf text;
 	bool intoken = false;
+	bool inWhitespace = false;
+	bool seeingSpace = false;
+	char curChar = '\0';
 
-	while (FileMgr::getLine(fd, buffer)) {
-		//cout << "Line: " << buffer.c_str() << endl;
-		for (from = buffer.c_str(); *from; from++) {
-			if (*from == '<') {
-				intoken = true;
-				token = "<";
-				continue;
-			}
+	while (infile.good()) {
+		
+		curChar = infile.get();
 
-			if (*from == '>') {
-				intoken = false;
-				token.append('>');
-				// take this isalpha if out to check for bugs in text
-				if ((isalpha(token[1])) || (isalpha(token[2]))) {
-					//cout << "Handle:" << token.c_str() << endl;
-					XMLTag *t = new XMLTag(token.c_str());
+		// skip the character if it is bad. infile.good() will catch the problem
+		if (curChar == -1) {
+			continue;
+		}
 
-					if (!handleToken(text, transform(t))) {
-						text.append(*t);
-					}
+		if (!intoken && curChar == '<') {
+			intoken = true;
+			token = "<";
+			continue;
+		}
+
+		// Outside of tokens merge adjacent whitespace
+		if (!intoken) {
+			seeingSpace = isspace(curChar);
+			if (seeingSpace) {
+				if (inWhitespace) {
+					continue;
 				}
-				continue;
+				// convert all whitespace to blanks
+				curChar = ' ';
 			}
+			inWhitespace = seeingSpace;
+		}
 
-			if (intoken)
-				token.append(*from);
-			else	
-				text.append(*from);
+		if (intoken && curChar == '>') {
+			intoken = false;
+			inWhitespace = false;
+			token.append('>');
+			// take this isalpha if out to check for bugs in text
+			if ((isalpha(token[1])) || (isalpha(token[2]))) {
+				//cout << "Handle:" << token.c_str() << endl;
+				XMLTag *t = new XMLTag(token.c_str());
+
+				if (!handleToken(text, transform(t))) {
+					text.append(*t);
+				}
+			}
+			continue;
 		}
 
 		if (intoken)
-			token.append("\n");
+			token.append(curChar);
+		else
+			switch (curChar) {
+				case '>' : text.append("&gt;"); break;
+				case '<' : text.append("&lt;"); break;
+				default  : text.append(curChar); break;
+			}
 	}
 
-	// Force the last entry from the buffer.
+	// Force the last entry from the text buffer.
 	text = "";
 	writeEntry(*currentVerse, text, true);
 	delete module;
 	delete currentVerse;
 	if (cipherFilter)
 		delete cipherFilter;
-	FileMgr::getSystemFileMgr()->close(fd);
+	infile.close();
 }
 
