@@ -1,6 +1,6 @@
 #include <iostream>
 #include <string>
-
+#include <sys/stat.h>
 #include <fcntl.h>
 
 #ifndef __GNUC__
@@ -9,13 +9,9 @@
 #include <unistd.h>
 #endif
 
+#include <filemgr.h>
 #include <lzsscomprs.h>
 
-#ifndef O_BINARY
-#define O_BINARY 0
-#endif
-
-
 using namespace std;
 #ifndef NO_SWORD_NAMESPACE
 using namespace sword;
@@ -79,7 +75,7 @@ int main(int argc, char **argv) {
 		bookpath += "/";
 
 	fileName = bookpath + "Book.dat";
-	int fd = open(fileName.c_str(), O_RDONLY|O_BINARY);
+	int fd = FileMgr::openFileReadOnly(fileName.c_str());
 
 	if (fd < 1) {
 		cerr << "error, couldn't open file: " << fileName << "\n";
@@ -92,7 +88,7 @@ int main(int argc, char **argv) {
 
 
 	fileName = bookpath + "Viewable.idx";
-	int fdv = open(fileName.c_str(), O_RDONLY|O_BINARY);
+	int fdv = FileMgr::openFileReadOnly(fileName.c_str());
 
 	if (fdv < 1) {
 		cerr << "error, couldn't open file: " << fileName << "\n";
